#pragma once

// too easy implementation
// no ref count, no other goodness, forgive me.
class CItemIDList;

// for debug
#ifdef _DEBUG
	const bool _Mtl_ItemIDList_traceOn = false;
	#define idlTRACE if (_Mtl_ItemIDList_traceOn) ATLTRACE
#else
	#define idlTRACE
#endif

class CItemIDList
{
public:

#ifdef _DEBUG
// for checking leaks
	static int d_m_nCount;
#define _INCREMENT_COUNT() ++CItemIDList::d_m_nCount; if (_Mtl_ItemIDList_traceOn) ATLTRACE(_T("CItemIDList(%d)\n"), CItemIDList::d_m_nCount)
#define _DECREMENT_COUNT() --CItemIDList::d_m_nCount; if (_Mtl_ItemIDList_traceOn) ATLTRACE(_T("CItemIDList(%d)\n"), CItemIDList::d_m_nCount)
#else
	#define _INCREMENT_COUNT()
	#define _DECREMENT_COUNT()
#endif

// Data members
	LPITEMIDLIST m_pidl;

// Constructors
	CItemIDList() : m_pidl(NULL)
	{
	}

	CItemIDList(const CItemIDList& idl)
	{
		m_pidl = _CopyItemIDList(idl);
	}

	CItemIDList(LPCITEMIDLIST pidl)
	{
		m_pidl = _CopyItemIDList(pidl);
	}

	CItemIDList(LPCTSTR lpszPath)// Note. this is late
	{
		m_pidl = NULL;

		ULONG chEaten;
		USES_CONVERSION;

		CComPtr<IShellFolder> spDesktopFolder;
		HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
		if (FAILED(hr))
			return;

		spDesktopFolder->ParseDisplayName(NULL, NULL, T2OLE((LPTSTR)lpszPath),
			&chEaten, &m_pidl, NULL);

		_INCREMENT_COUNT();
	}

// Destructor
	~CItemIDList()
	{
		_Release(m_pidl);
	}

	void Release()
	{
		_Release(m_pidl);
		m_pidl = NULL;
	}

	static void FreeIDList(LPITEMIDLIST pidl)
	{
		_Release(pidl);
	}

// Attributes
	bool IsNull()
	{
		return (m_pidl == NULL);
	}

	CString GetPath()
	{
		TCHAR szPath[MAX_PATH];
		if (::SHGetPathFromIDList(m_pidl, szPath))
			return szPath;
		else
			return CString();
	}

	void Attach(LPITEMIDLIST pidl)
	{
		ATLASSERT(m_pidl == NULL);
		m_pidl = pidl;
	}

	LPITEMIDLIST Detach()
	{
		LPITEMIDLIST idl = m_pidl;
		m_pidl = NULL;
		return idl;
	}

	operator LPITEMIDLIST() const
	{
		return m_pidl;
	}

//	operator LPCITEMIDLIST() const
//	{
//		return m_pidl;
//	}

	LPITEMIDLIST* operator&()
	{
		ATLASSERT(m_pidl == NULL);
		_INCREMENT_COUNT();
		return &m_pidl;
	}

	UINT GetSize()
	{
		return _GetSize(m_pidl);
	}

// Operations
/*	bool operator==(const CItemIDList& idl) const
	{
		return _CmpItemID(m_pidl, idl);
	}

	bool operator!=(const CItemIDList& idl) const
	{
		return !_CmpItemID(m_pidl, idl);
	}
*/
	void operator+=(const CItemIDList& idl)
	{
		LPITEMIDLIST pidlNew = _ConcatPidls(m_pidl, idl);
		Release();
		m_pidl = pidlNew;
	}

	void operator-=(const CItemIDList& idl)
	{
		LPITEMIDLIST pidlNew = _RelativePidls(m_pidl, idl);
		Release();
		m_pidl = pidlNew;
	}

	bool Find(LPCTSTR lpszText)
	{
		int nLen = ::lstrlen(lpszText);
		if (nLen == 0)
			return false;

		int nSrc = _GetSize(m_pidl);
		if (nSrc < nLen)
			return false;

		const BYTE* p = (BYTE*)m_pidl;

		for (int i = 0; i < nSrc - nLen; ++i) {
			if(::memcmp(p + i, lpszText, nLen * sizeof(TCHAR)) == 0) {
				return true;
			}
		}

		return false;
	}

// Operators
	CItemIDList& operator=(const CItemIDList& idlSrc)
	{
		Release();
		m_pidl = _CopyItemIDList(idlSrc);
		return *this;
	}
	
	CItemIDList& operator=(LPCTSTR lpszPath)
	{
		Release();

		CComPtr<IShellFolder> spDesktopFolder;
		HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
		if (FAILED(hr))
			return *this;

		ULONG chEaten;
		USES_CONVERSION;
		hr = spDesktopFolder->ParseDisplayName(NULL, NULL, T2OLE((LPTSTR)lpszPath),
			&chEaten, &m_pidl, NULL);
		
		_INCREMENT_COUNT();
		return *this;
	}

	CItemIDList operator+(const CItemIDList& idlSrc) const
	{
		CItemIDList idl;
		idl.m_pidl = _ConcatPidls(m_pidl, idlSrc);
		return idl;
	}

	CItemIDList operator-(const CItemIDList& idlSrc) const
	{
		CItemIDList idl;
		idl.m_pidl = _RelativePidls(m_pidl, idlSrc);
		return idl;
	}

// Helpers
	static LPITEMIDLIST _Create(UINT cbSize)
	{
		LPITEMIDLIST pidl = (LPITEMIDLIST)::CoTaskMemAlloc(cbSize);
		if (pidl == NULL)
			return NULL;

		_INCREMENT_COUNT();

		if (pidl)
			::memset(pidl, 0, cbSize);		// zero-init for external task	 alloc

		return pidl;
	}

	static void _Release(LPITEMIDLIST pidl)
	{
#ifdef _DEBUG
		if (pidl != NULL) {
			_DECREMENT_COUNT();
		}
#endif

		::CoTaskMemFree(pidl);
	}

	static UINT _GetSize(LPCITEMIDLIST pidl)
	{// include null terminator
		UINT cbTotal = 0;
		if (pidl) {
			cbTotal += sizeof(pidl->mkid.cb);		// Null terminator
			while (pidl->mkid.cb) {
				cbTotal += pidl->mkid.cb;
				pidl = _Next(pidl);
			}
		}

		return cbTotal;
	}

	static LPITEMIDLIST _Next(LPCITEMIDLIST pidl)
	{
		ATLASSERT(pidl != NULL);

		LPSTR lpMem=(LPSTR)pidl;
		lpMem += pidl->mkid.cb;
		return (LPITEMIDLIST)lpMem;
	}

	static LPITEMIDLIST _ConcatPidls(LPCITEMIDLIST pidl1, LPCITEMIDLIST pidl2)
	{
		UINT cb1;
		if (pidl1)	//May be NULL
		   cb1 = _GetSize(pidl1) - sizeof(pidl1->mkid.cb);
		else
		   cb1 = 0;

		UINT cb2 = _GetSize(pidl2);	

		LPITEMIDLIST pidlNew = _Create(cb1 + cb2);
		if (pidlNew) {
			if (pidl1)
				::memcpy(pidlNew, pidl1, cb1);
			::memcpy(((LPSTR)pidlNew) + cb1, pidl2, cb2);
			ATLASSERT(_GetSize(pidlNew) == cb1 + cb2);
		}

		return pidlNew;
	}

	static LPITEMIDLIST _RelativePidls(LPCITEMIDLIST pidl1, LPCITEMIDLIST pidl2)
	{
		ATLASSERT(pidl1 != NULL);

		UINT cb1 = _GetSize(pidl1);

		UINT cb2;
		if (pidl2)	//May be NULL
		   cb2 = _GetSize(pidl2) - sizeof(pidl2->mkid.cb);
		else
		   cb2 = 0;

		if (cb1 <= cb2)
			return NULL;
		
		LPITEMIDLIST pidlNew = _Create(cb1 - cb2);
		if (pidlNew) {
			if (pidl1)
				::memcpy(pidlNew, ((LPSTR)pidl1) + cb2, cb1 - cb2);

			ATLASSERT(_GetSize(pidlNew) == cb1 - cb2);
		}

		return pidlNew;
	}


	static LPITEMIDLIST _CopyItemIDList(LPCITEMIDLIST lpi)
	{
		if (lpi == NULL)
			return NULL;

		UINT cb = _GetSize(lpi);
		LPITEMIDLIST lpiTemp = (LPITEMIDLIST)::CoTaskMemAlloc(cb);
		if (lpiTemp == NULL)
			return NULL;
		
		_INCREMENT_COUNT();

		::memcpy(lpiTemp, lpi, cb);

		return lpiTemp;
	}

/*	static LPITEMIDLIST _CopyItemID(LPCITEMIDLIST lpi)
	{
		if (lpi == NULL)
			return NULL;

		LPITEMIDLIST lpiTemp = (LPITEMIDLIST)::CoTaskMemAlloc( lpi->mkid.cb + sizeof(lpi->mkid.cb) );
		if (lpiTemp == NULL)
			return NULL;

		_INCREMENT_COUNT();

		::memcpy(lpiTemp, lpi, lpi->mkid.cb + sizeof(lpi->mkid.cb));

		return lpiTemp;
	}
*/
	static bool _CmpItemID(LPCITEMIDLIST pidl1, LPCITEMIDLIST pidl2)
	{
		if (pidl1 == NULL && pidl2 != NULL)
			return false;

		if (pidl1 != NULL && pidl2 == NULL)
			return false;

		UINT cb1 = _GetSize(pidl2);
		UINT cb2 = _GetSize(pidl1);
		
		if (cb1 != cb2)
			return false;

		return ::memcmp(pidl1, pidl2, cb1) == 0;
	}

	static void _Dump(LPCITEMIDLIST pidlSrc)
	{
		if  (pidlSrc == NULL)
			return;

		LPITEMIDLIST pidl = (LPITEMIDLIST)pidlSrc;
		while (pidl->mkid.cb) {
			idlTRACE(_T(" "));
			for (int i = sizeof(pidl->mkid.cb); i < pidl->mkid.cb; ++i)
				idlTRACE(_T("%c"), *((LPBYTE)pidl + i));
			pidl = _Next(pidl);
		}

		idlTRACE(_T("\n"));
	}
};

#ifdef _DEBUG
	__declspec(selectany) int CItemIDList::d_m_nCount = 0;
#endif

// Compare helpers
inline bool operator==(const CItemIDList& s1, const CItemIDList& s2)
{
	return CItemIDList::_CmpItemID(s1, s2);

}
inline bool operator==(const CItemIDList& s1, LPCITEMIDLIST s2)
{
	return CItemIDList::_CmpItemID(s1, s2);
}

inline bool operator==(LPCITEMIDLIST s1, const CItemIDList& s2)
{
	return CItemIDList::_CmpItemID(s1, s2);
}

inline bool operator!=(const CItemIDList& s1, const CItemIDList& s2)
{
	return !CItemIDList::_CmpItemID(s1, s2);
}

inline bool operator!=(const CItemIDList& s1, LPCITEMIDLIST s2)
{
	return !CItemIDList::_CmpItemID(s1, s2);
}

inline bool operator!=(LPCITEMIDLIST s1, const CItemIDList& s2)
{
	return !CItemIDList::_CmpItemID(s1, s2);
}

/*
template <class _Function>
_Function MtlForEachFile(IShellFolder* pFolder, _Function __f, bool bIncludeHidden = false)
{
	DWORD grfFlags = bIncludeHidden ? SHCONTF_NONFOLDERS|SHCONTF_INCLUDEHIDDEN : SHCONTF_NONFOLDERS;

	CComPtr<IEnumIDList> spEnum;
	HRESULT hr = pFolder->EnumObjects(NULL, grfFlags, &spEnum);
	if (FAILED(hr))
		return __f;

	LPITEMIDLIST pidl;
	ULONG celtFetched;
	while (spEnum->Next(1, &pidl, &celtFetched) == S_OK) {
		ATLASSERT(pidl != NULL);
		CItemIDList idl; idl.Attach(pidl);
		__f(pFolder, idl);
	}

	return __f;
}

template <typename _Function>
_Function MtlForEachFolder(IShellFolder* pFolder, _Function __f, bool bIncludeHidden = false)
{
	DWORD grfFlags = bIncludeHidden ? SHCONTF_FOLDERS|SHCONTF_INCLUDEHIDDEN : SHCONTF_FOLDERS;

	CComPtr<IEnumIDList> spEnum;
	HRESULT hr = pFolder->EnumObjects(NULL, grfFlags, &spEnum);
	if (FAILED(hr))
		return __f;

	LPITEMIDLIST pidl;
	ULONG celtFetched;
	while (spEnum->Next(1, &pidl, &celtFetched) == S_OK) {
		ATLASSERT(pidl != NULL);
		CItemIDList idl; idl.Attach(pidl);
		__f(pFolder, idl);
	}

	return __f;
}

template <typename _Function>
_Function MtlForEachFolderAndFile(IShellFolder* pFolder, _Function __f, bool bIncludeHidden = false)
{
	DWORD grfFlags = bIncludeHidden ? SHCONTF_NONFOLDERS|SHCONTF_FOLDERS|SHCONTF_INCLUDEHIDDEN : SHCONTF_NONFOLDERS|SHCONTF_FOLDERS;

	CComPtr<IEnumIDList> spEnum;
	HRESULT hr = pFolder->EnumObjects(NULL, grfFlags, &spEnum);
	if (FAILED(hr))
		return __f;

	LPITEMIDLIST pidl;
	ULONG celtFetched;
	while (spEnum->Next(1, &pidl, &celtFetched) == S_OK) {
		_INCREMENT_COUNT();
		ATLASSERT(pidl != NULL);
		CItemIDList idl; idl.Attach(pidl);
		__f(pFolder, idl);
	}

	return __f;
}
	*/

template <class _Function>
_Function MtlForEachObject(const CItemIDList& idlFolder, _Function __f, bool bIncludeHidden = false)
{
	// get desktop
	CComPtr<IShellFolder> spDesktopFolder;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
	if (FAILED(hr))
		return __f;

	// get the folder
	CComPtr<IShellFolder> spFolder;
	hr = spDesktopFolder->BindToObject(idlFolder, NULL, IID_IShellFolder, (void**)&spFolder);
	if (FAILED(hr)) {
		CItemIDList idl;
		hr = ::SHGetSpecialFolderLocation(NULL, CSIDL_DESKTOP, &idl);
		if (FAILED(hr))
			return __f;

		if (idl == idlFolder)// check if same idl or not
			spFolder = spDesktopFolder;
		else	
			return __f;
	}

	DWORD grfFlags = SHCONTF_FOLDERS|SHCONTF_NONFOLDERS;
	if (bIncludeHidden)
		grfFlags |= SHCONTF_INCLUDEHIDDEN;

	CComPtr<IEnumIDList> spEnum;
	hr = spFolder->EnumObjects(NULL, grfFlags, &spEnum);
	if (FAILED(hr))
		return __f;

	LPITEMIDLIST pidl;
	ULONG celtFetched;
	while (spEnum->Next(1, &pidl, &celtFetched) == S_OK) {
		_INCREMENT_COUNT();
		ATLASSERT(pidl != NULL);

		CItemIDList idlFile; idlFile.Attach(pidl);

		ULONG dwAttributes =  SFGAO_FOLDER;
		LPCITEMIDLIST pidls[] = { idlFile.m_pidl };
		HRESULT hr = spFolder->GetAttributesOf(1, pidls, &dwAttributes);
		if (FAILED(hr))
			continue;

		bool bFolder = _check_flag(SFGAO_FOLDER, dwAttributes);// check
		__f(spFolder, idlFolder, idlFile, bFolder);
	}

	return __f;
}

template <class _Function>
_Function MtlForEachFolder(const CItemIDList& idlFolder, _Function __f, bool bIncludeHidden = false)
{
	// get desktop
	CComPtr<IShellFolder> spDesktopFolder;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
	if (FAILED(hr))
		return __f;

	// get the folder
	CComPtr<IShellFolder> spFolder;
	hr = spDesktopFolder->BindToObject(idlFolder, NULL, IID_IShellFolder, (void**)&spFolder);
	if (FAILED(hr))
		return __f;

	DWORD grfFlags = bIncludeHidden ? SHCONTF_FOLDERS|SHCONTF_INCLUDEHIDDEN : SHCONTF_FOLDERS;

	CComPtr<IEnumIDList> spEnum;
	hr = spFolder->EnumObjects(NULL, grfFlags, &spEnum);
	if (FAILED(hr))
		return __f;

	LPITEMIDLIST pidl;
	ULONG celtFetched;
	while (spEnum->Next(1, &pidl, &celtFetched) == S_OK) {
		_INCREMENT_COUNT();
		ATLASSERT(pidl != NULL);

		CItemIDList idlFile; idlFile.Attach(pidl);

		ULONG dwAttributes = SFGAO_FOLDER;
		LPCITEMIDLIST pidls[] = { idlFile.m_pidl };
		spFolder->GetAttributesOf(1, pidls, &dwAttributes);
		if (dwAttributes & SFGAO_FOLDER)	// required
			__f(spFolder, idlFolder, idlFile);
	}

	return __f;
}

template <class _Function>
_Function MtlForEachFile(const CItemIDList& idlFolder, _Function __f, bool bIncludeHidden = false)
{
	// get desktop
	CComPtr<IShellFolder> spDesktopFolder;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
	if (FAILED(hr))
		return __f;

	// get the folder
	CComPtr<IShellFolder> spFolder;
	hr = spDesktopFolder->BindToObject(idlFolder, NULL, IID_IShellFolder, (void**)&spFolder);
	if (FAILED(hr))
		return __f;

	DWORD grfFlags = bIncludeHidden ? SHCONTF_NONFOLDERS|SHCONTF_INCLUDEHIDDEN : SHCONTF_NONFOLDERS;

	CComPtr<IEnumIDList> spEnum;
	hr = spFolder->EnumObjects(NULL, grfFlags, &spEnum);
	if (FAILED(hr))
		return __f;

	LPITEMIDLIST pidl;
	ULONG celtFetched;
	while (spEnum->Next(1, &pidl, &celtFetched) == S_OK) {
		_INCREMENT_COUNT();
		ATLASSERT(pidl != NULL);

		CItemIDList idlFile; idlFile.Attach(pidl);

		ULONG dwAttributes = SFGAO_FOLDER;
		LPCITEMIDLIST pidls[] = { idlFile.m_pidl };
		spFolder->GetAttributesOf(1, pidls, &dwAttributes);
		if (!(dwAttributes & SFGAO_FOLDER))	// required
			__f(spFolder, idlFolder, idlFile);
	}

	return __f;
}

inline bool MtlGetDisplayName(IShellFolder* pFolder, LPCITEMIDLIST lpi,
							  CString& rString, DWORD uFlags = SHGDN_NORMAL)
{
	ATLASSERT(pFolder != NULL);
	ATLASSERT(lpi != NULL);

	STRRET str;
	HRESULT hr = pFolder->GetDisplayNameOf(lpi, uFlags, &str);
	if (FAILED(hr))
		return false;

	USES_CONVERSION;
	switch (str.uType) {
	case STRRET_WSTR:
		rString = W2T(str.pOleStr);
		break;

	case STRRET_OFFSET:
		rString = (LPSTR)lpi + str.uOffset;
		break;

	case STRRET_CSTR:             
		rString = (LPSTR)str.cStr;
		break;
	
	default:
		 return false;
	}

	return true;
}
/*
inline CString MtlGetDisplayName(const CString& strFullPath)
{
	CComPtr<IShellFolder> spDesktop;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktop);
	if (FAILED(hr))
		return CString();

	CString strName;
	if (!MtlGetDisplayName(spDesktop, CItemIDList(strFullPath), strName))
		return CString();

	return strName;
}
*/
inline CString MtlGetDisplayName(LPCITEMIDLIST pidlRelative)
{// The desktop folder can give a name to you even if a relative idl
	if (pidlRelative == NULL)
		return CString();

	CComPtr<IShellFolder> spDesktop;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktop);
	if (FAILED(hr))
		return CString();

	CString strName;
	if (!MtlGetDisplayName(spDesktop, pidlRelative, strName))
		return CString();

	return strName;
}

inline CItemIDList MtlSetDisplayName(IShellFolder* pFolder, LPCITEMIDLIST pidl,
									 LPCTSTR lpszName, HWND hWnd, DWORD uFlags = SHGDN_NORMAL)
{
	CItemIDList idl;
	USES_CONVERSION;
	HRESULT hr = pFolder->SetNameOf(hWnd, pidl, T2COLE(lpszName), uFlags, &idl);
	if (SUCCEEDED(hr))
		return idl;

	return idl;
}
/*
inline bool MtlGetDisplayName(CString& strName, const CString& strFolder, const CString& strObject)
{
	CItemIDList idlFolder = strFolder;
	CItemIDList idl = strObject;
	idl -= idlFolder;

	if (idl.IsNull())
		return false;

	CComPtr<IShellFolder> spDesktop;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktop);
	if (FAILED(hr))
		return false;

	CComPtr<IShellFolder> spFolder;
	HRESULT hr = spDesktopFolder->BindToObject(idlFolder, NULL, IID_IShellFolder, (void**)&spFolder);
	if (FAILED(hr))
		return false;

	return MtlGetDisplayName(spFolder, idl, strName);
}
*/
bool MtlGetShellFolder(LPCITEMIDLIST pidl, IShellFolder** ppFolder)
{
	ATLASSERT(pidl != NULL);

	CComPtr<IShellFolder> spDesktop;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktop);
	if (FAILED(hr))
		return false;

	hr = spDesktop->BindToObject(pidl, NULL, IID_IShellFolder, (void**)ppFolder);
	if (FAILED(hr))
		return false;

	return true;
}

// Note. It would fail if not the ordinary path.
inline CItemIDList MtlGetFullItemIDList(IShellFolder* pFolder, LPCITEMIDLIST lpi)
{
	CString strName;
	if (!MtlGetDisplayName(pFolder, lpi, strName, SHGDN_FORPARSING))
		return CItemIDList();

	CComPtr<IShellFolder> spDesktopFolder;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
	if (FAILED(hr))
		return CItemIDList();

	ULONG chEaten;
	CItemIDList idlFolder;
	USES_CONVERSION;
	hr = spDesktopFolder->ParseDisplayName(NULL, NULL, T2OLE((LPTSTR)(LPCTSTR)strName),
			&chEaten, &idlFolder, NULL);
	if (FAILED(hr))
		return CItemIDList();

	return idlFolder;
}

inline int MtlGetSystemIconIndex(LPCITEMIDLIST lpi, UINT uFlags = SHGFI_SMALLICON)
{
	ATLASSERT(lpi != NULL);

	uFlags |= SHGFI_PIDL | SHGFI_SYSICONINDEX;
				
	SHFILEINFO sfi;   
	DWORD dwResult = ::SHGetFileInfo((LPCTSTR)lpi, 0, &sfi, sizeof(SHFILEINFO), uFlags);
	if (dwResult == 0)
		return -1;

	return sfi.iIcon;
}


inline bool MtlGetAttributesOf(LPCITEMIDLIST pidlFull, ULONG& dwAttributes)
{
	CComPtr<IShellFolder> spDesktopFolder;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
	if (FAILED(hr))
		return false;

	LPCITEMIDLIST pidls[] = { pidlFull };
	hr = spDesktopFolder->GetAttributesOf(1, pidls, &dwAttributes);
	if (FAILED(hr))
		return false;

	return true;
}

// Note. It is possible to make browsing slow.
inline bool MtlHasSubObject(const CItemIDList& idlFolder, bool bIncludeHidden = false)
{
	ATLASSERT(idlFolder.m_pidl != NULL);
	// get desktop
	CComPtr<IShellFolder> spDesktopFolder;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
	if (FAILED(hr))
		return false;

	// get the folder
	CComPtr<IShellFolder> spFolder;
	hr = spDesktopFolder->BindToObject(idlFolder, NULL, IID_IShellFolder, (void**)&spFolder);
	if (FAILED(hr))
		return false;

	DWORD grfFlags = bIncludeHidden ? SHCONTF_FOLDERS|SHCONTF_NONFOLDERS|SHCONTF_INCLUDEHIDDEN : SHCONTF_FOLDERS|SHCONTF_NONFOLDERS;

	CComPtr<IEnumIDList> spEnum;
	hr = spFolder->EnumObjects(NULL, grfFlags, &spEnum);
	if (FAILED(hr))
		return false;

	LPITEMIDLIST pidl;
	ULONG celtFetched;
	while (spEnum->Next(1, &pidl, &celtFetched) == S_OK) {
		ATLASSERT(pidl != NULL);
		CItemIDList idl; idl.Attach(pidl);
		return true;
	}

	return false;
}

// Note. this is still not precise.
inline bool MtlIsRealFolder(const CItemIDList& idlFolder)
{
	ATLASSERT(idlFolder.m_pidl != NULL);
	// get desktop
	CComPtr<IShellFolder> spDesktopFolder;
	HRESULT hr = ::SHGetDesktopFolder(&spDesktopFolder);
	if (FAILED(hr))
		return false;

	ULONG dwAttributes = SFGAO_FOLDER;
	LPCITEMIDLIST pidls[] = { idlFolder.m_pidl };
	hr = spDesktopFolder->GetAttributesOf(1, pidls, &dwAttributes);
	if (dwAttributes & SFGAO_FOLDER)
		return true;
	else
		return false;
}

bool MtlShellExecute(HWND hWnd, LPCITEMIDLIST pidl)
{
	ATLASSERT(pidl != NULL);

	SHELLEXECUTEINFO sei = {
		sizeof(SHELLEXECUTEINFO), SEE_MASK_INVOKEIDLIST,
		hWnd,
		_T(""), _T(""), _T(""),	_T(""),
		SW_SHOWNORMAL, _Module.GetModuleInstance(),
		(LPVOID)pidl,
		NULL, 0, 0, NULL
	};

	return ShellExecuteEx(&sei) == TRUE;
}

inline bool MtlIsTooSlowIDList(LPCITEMIDLIST pidl)
{
	TCHAR szPath[MAX_PATH]; szPath[0] = _T('\0');
	::SHGetPathFromIDList(pidl, szPath);
	CString strExt = CString(szPath).Right(4);
	return (strExt.CompareNoCase(_T(".url")) == 0);
}

/*
template <typename _OutputIter>
bool MtlEnumFile(IShellFolder* pFolder, _OutputIter __result, bool bIncludeHidden = false)
{
  *__result = *__first;
  while (++__first != __last)
    if (!(*__result == *__first))
      *++__result = *__first;
  return ++__result;

	DWORD grfFlags = bIncludeHidden ? SHCONTF_NONFOLDERS|SHCONTF_INCLUDEHIDDEN : SHCONTF_NONFOLDERS;

	CComPtr<IEnumIDList> spEnum;
	HRESULT hr = pFolder->EnumObjects(NULL, grfFlags, &spEnum);
	if (FAILED(hr))
		return __f;

	typedef enum_iterator<IEnumIDList, &IID_IEnumIDList, LPITEMIDLIST> pidl_itor;
	for (pidl_itor it = pidl_itor(spEnum, 64); it != pidl_itor(); ++it) {
		CItemIDList idl; idl.Attach(*it);
		__f(pFolder, idl);
	}

	return __f;
}
*/