#ifndef __MTLUSER_H__
#define __MTLUSER_H__

// MTL Version 0.03
// Copyright (C) 2000 MB<mb2@geocities.co.jp>
// All rights unreserved.
//
// This file is a part of Mb Template Library.
// The code and information is *NOT* provided "as-is" without
// warranty of any kind, either expressed or implied.
//
// Last updated: August 11, 2000

#pragma once

#ifndef __MTLMISC_H__
	#error mtluser.h requires mtlmisc.h to be included first
#endif

#ifndef __MTLWIN_H__
	#error mtluser.h requires mtlwin.h to be included first
#endif

#include <functional>
#include <zmouse.h>

namespace MTL
{
	using WTL::CReBarCtrl;

////////////////////////////////////////////////////////////////////////////
// for ReBar Bands

inline BOOL MtlToggleBandVisible(CReBarCtrl rebar, UINT nID)
{
	ATLASSERT(rebar.IsWindow());

	int nIndex = rebar.IdToIndex(nID);
	ATLASSERT(nIndex != -1);

	CVersional<REBARBANDINFO> rbBand;
	rbBand.fMask = RBBIM_STYLE;
	MTLVERIFY(rebar.GetBandInfo(nIndex, &rbBand));

	BOOL bNew = (rbBand.fStyle & RBBS_HIDDEN) ? TRUE : FALSE;
	MTLVERIFY(rebar.ShowBand(nIndex, bNew));
	return bNew;
}

inline BOOL MtlShowBand(CReBarCtrl rebar, UINT nID, BOOL bVisible = TRUE)
{
	ATLASSERT(rebar.IsWindow());

	int nIndex = rebar.IdToIndex(nID);
	ATLASSERT(nIndex != -1);

	CVersional<REBARBANDINFO> rbBand;
	rbBand.fMask = RBBIM_STYLE;
	MTLVERIFY(rebar.GetBandInfo(nIndex, &rbBand));

	BOOL bOldVisible = (rbBand.fStyle & RBBS_HIDDEN) ? FALSE : TRUE;

	if (bOldVisible != bVisible)
		MTLVERIFY(rebar.ShowBand(nIndex, bVisible));

	return TRUE;
}

inline BOOL MtlIsBandVisible(CReBarCtrl rebar, UINT nID)
{
	ATLASSERT(rebar.IsWindow());

	int nIndex = rebar.IdToIndex(nID);
	ATLASSERT(nIndex != -1);

	CVersional<REBARBANDINFO> rbBand;
	rbBand.fMask = RBBIM_STYLE;
	MTLVERIFY(rebar.GetBandInfo(nIndex, &rbBand));

	return rbBand.fStyle & RBBS_HIDDEN ? FALSE : TRUE;
}

#define REFLECT_CHEVRONPUSHED_NOTIFICATION() \
	{ \
		if (uMsg == WM_NOTIFY && RBN_CHEVRONPUSHED == ((LPNMHDR)lParam)->code) \
		{ \
			bHandled = TRUE; \
			LPNMREBARCHEVRON lpnm = (LPNMREBARCHEVRON)lParam; \
			HWND hWndReBar = lpnm->hdr.hwndFrom; \
			HWND hWndBand = MtlGetReBarBandHwnd(hWndReBar, lpnm->wID); \
			if (hWndBand != NULL) \
				lResult = ::SendMessage(hWndBand, OCM__BASE + uMsg, wParam, lParam); \
			else \
				bHandled = FALSE; \
			if (bHandled) \
				return TRUE; \
		} \
	}

inline HWND MtlGetReBarBandHwnd(CReBarCtrl rebar, UINT nID)
{
	ATLASSERT(rebar.IsWindow());

	int nIndex = rebar.IdToIndex(nID);
	ATLASSERT(nIndex != -1);

	CVersional<REBARBANDINFO> rbBand;
	rbBand.fMask = RBBIM_CHILD;
	MTLVERIFY(rebar.GetBandInfo(nIndex, &rbBand));
	ATLASSERT(::IsWindow(rbBand.hwndChild));
	return rbBand.hwndChild;
}

////////////////////////////////////////////////////////////////////////////
// UIEnable and UISetCheck, UISetRadio

class UIEnableFunction : public std::unary_function<int, BOOL>
{
public:
	explicit UIEnableFunction(CUpdateUIBase* pUpdateUI, BOOL bEnable)
		: _pUpdateUI(pUpdateUI), _bEnable(bEnable) { }
	result_type operator()(argument_type arg)
	{
		return _pUpdateUI->UIEnable(arg, _bEnable);
	}

private:
	CUpdateUIBase* _pUpdateUI;
	BOOL _bEnable;
};

class UISetCheckFunction : public std::unary_function<int, BOOL>
{
public:
	explicit UISetCheckFunction(CUpdateUIBase* pUpdateUI, int nCheck)
		: _pUpdateUI(pUpdateUI), _nCheck(nCheck) { }
	result_type operator()(argument_type arg)
	{
		return _pUpdateUI->UISetCheck(arg, _nCheck);
	}

private:
	CUpdateUIBase* _pUpdateUI;
	int _nCheck;
};


class UISetRadioFunction : public std::unary_function<int, BOOL>
{
public:
	explicit UISetRadioFunction(CUpdateUIBase* pUpdateUI, BOOL bRadio)
		: _pUpdateUI(pUpdateUI), _bRadio(bRadio) { }

	result_type operator()(argument_type arg)
	{
		return _pUpdateUI->UISetRadio(arg, _bRadio);
	}

private:
	CUpdateUIBase* _pUpdateUI;
	BOOL _bRadio;
};

////////////////////////////////////////////////////////////////////////////
// for standard application commands

template <class T>
class CAppCommandHandler
{
public:
	BEGIN_MSG_MAP(CAppCommandHandler)
		COMMAND_ID_HANDLER(ID_APP_EXIT, OnAppExit)
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
		COMMAND_ID_HANDLER(ID_APP_HELP, OnAppHelp) // UDT DGSTR
	END_MSG_MAP()

	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		CAboutDlg dlg;
		dlg.DoModal();
		return 0;
	}

	LRESULT OnAppExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(::IsWindow(pT->m_hWnd));
		pT->PostMessage(WM_CLOSE);
		return 0;
	}

	// UDT DGSTR ( ShowHelp )
	LRESULT OnAppHelp(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{		
		CString strPath = MtlGetHTMLHelpPath();
		// For compacting size , dont use htmlHelp api.
		ShellExecute(NULL , NULL , strPath , NULL , NULL , SW_SHOW );
		return 0;
	}
	// ENDE
};


////////////////////////////////////////////////////////////////////////////
// for Window Menu command

struct _Mtl_Function_CloseAll
{
	void operator()(HWND hWnd)
	{
		::SendMessage(hWnd, WM_CLOSE, 0, 0);
	}
};

struct _Mtl_Function_CloseAllExcept
{
	HWND _hWndExcept;
	_Mtl_Function_CloseAllExcept(HWND hWndExcept) : _hWndExcept(hWndExcept) { }
	void operator()(HWND hWnd)
	{
		if (hWnd != _hWndExcept)
			::SendMessage(hWnd, WM_CLOSE, 0, 0);
	}
};

struct _Mtl_Function_CloseAllExcept2
{
	HWND _hWndExcept, _hWndExcept2;
	_Mtl_Function_CloseAllExcept2(HWND hWndExcept, HWND hWndExcept2)
		: _hWndExcept(hWndExcept), _hWndExcept2(hWndExcept2) { }
	void operator()(HWND hWnd)
	{
		if (hWnd != _hWndExcept && hWnd != _hWndExcept2)
			::SendMessage(hWnd, WM_CLOSE, 0, 0);
	}
};

inline void MtlCloseAllMDIChildren(HWND hWndMDIClient)
{
	HWND hWndActive =(HWND)::SendMessage(hWndMDIClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL);
	MtlForEachMDIChild(hWndMDIClient, _Mtl_Function_CloseAllExcept(hWndActive));
	::SendMessage(hWndActive, WM_CLOSE, 0, 0);
}

inline void MtlCloseAllMDIChildrenExcept(HWND hWndMDIClient, HWND hWndExcept)
{
	HWND hWndActive =(HWND)::SendMessage(hWndMDIClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL);
	if (hWndActive == hWndExcept) {
		MtlForEachMDIChild(hWndMDIClient, _Mtl_Function_CloseAllExcept(hWndExcept));
	}
	else {
		MtlForEachMDIChild(hWndMDIClient, _Mtl_Function_CloseAllExcept2(hWndActive, hWndExcept));
		::SendMessage(hWndActive, WM_CLOSE, 0, 0);
	}
}

inline void MtlCompactMDIWindowMenuDocumentList(CMenuHandle menuWindow, int nWindowMenuItemCount, int nMaxTextLength)
{
	int nLen = nMaxTextLength + 5;// add a little
	LPTSTR lpszText = (LPTSTR)_alloca((nLen+1)*sizeof(TCHAR));
	int i = nWindowMenuItemCount + 1;// add a separator
	for (; i < menuWindow.GetMenuItemCount(); ++i) {
		CMenuItemInfo mii;
		mii.fMask = MIIM_TYPE;
		mii.cch = nLen;
		mii.dwTypeData = lpszText;
		::GetMenuItemInfo(menuWindow, i, TRUE, &mii);

		CString strMenuItem(mii.dwTypeData);
		int nHeaderLen = strMenuItem.Left(1) == _T("&") ? 3 : 0;
		CString strHeader = strMenuItem.Left(nHeaderLen);
		strMenuItem = MtlCompactString(strMenuItem.Right(strMenuItem.GetLength() - nHeaderLen), nMaxTextLength);
		strMenuItem = strHeader + strMenuItem;
		mii.dwTypeData = (LPTSTR)(LPCTSTR)strMenuItem;
		::SetMenuItemInfo(menuWindow, i, TRUE, &mii);
	}
}

template <class T>
class CWindowMenuCommandHandler
{
public:
// Message map and handlers
	BEGIN_MSG_MAP(CWindowMenuCommandHandler)
		COMMAND_ID_HANDLER(ID_WINDOW_CASCADE, OnWindowCascade)
		COMMAND_ID_HANDLER(ID_WINDOW_TILE_HORZ, OnWindowTileHorz)
		COMMAND_ID_HANDLER(ID_WINDOW_TILE_VERT, OnWindowTileVert)
		COMMAND_ID_HANDLER(ID_WINDOW_ARRANGE, OnWindowArrangeIcons)
		COMMAND_ID_HANDLER(ID_WINDOW_CLOSE_ALL, OnWindowCloseAll)
//		COMMAND_ID_HANDLER(ID_WINDOW_CLOSE_EXCEPT, OnWindowCloseExcept)

		COMMAND_ID_HANDLER(ID_WINDOW_RESTORE, OnWindowRestore)
		COMMAND_ID_HANDLER(ID_WINDOW_MOVE, OnWindowMove)
		COMMAND_ID_HANDLER(ID_WINDOW_SIZE, OnWindowSize)
		COMMAND_ID_HANDLER(ID_WINDOW_MINIMIZE, OnWindowMinimize)
		COMMAND_ID_HANDLER(ID_WINDOW_MAXIMIZE, OnWindowMaximize)
	END_MSG_MAP()

	LRESULT OnWindowCascade(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(::IsWindow(pT->m_hWnd));
		pT->MDICascade();
		return 0;
	}

	LRESULT OnWindowTileHorz(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(::IsWindow(pT->m_hWnd));
		pT->MDITile();
		return 0;
	}

	LRESULT OnWindowTileVert(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(::IsWindow(pT->m_hWnd));
		pT->MDITile(MDITILE_VERTICAL);
		return 0;
	}

	LRESULT OnWindowArrangeIcons(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(::IsWindow(pT->m_hWnd));
		pT->MDIIconArrange();
		return 0;
	}

	LRESULT OnWindowCloseAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(::IsWindow(pT->m_hWnd));
		ATLASSERT(::IsWindow(pT->m_hWndMDIClient));

		CWaitCursor cur;
		CLockRedrawMDIClient lock(pT->m_hWndMDIClient);
		MtlCloseAllMDIChildren(pT->m_hWndMDIClient);
		return 0;
	}

	LRESULT OnWindowCloseExcept(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(::IsWindow(pT->m_hWnd));
		ATLASSERT(::IsWindow(pT->m_hWndMDIClient));

		CWaitCursor cur;
		CLockRedrawMDIClient lock(pT->m_hWndMDIClient);
		MtlCloseAllMDIChildrenExcept(pT->m_hWndMDIClient, pT->MDIGetActive());
		return 0;
	}

	LRESULT OnWindowRestore(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		_SendSysCommand(SC_RESTORE);
		return 0;
	}

	LRESULT OnWindowMove(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		_SendSysCommand(SC_MOVE);
		return 0;
	}

	LRESULT OnWindowSize(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		_SendSysCommand(SC_SIZE);
		return 0;
	}

	LRESULT OnWindowMinimize(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		_SendSysCommand(SC_MINIMIZE);
		return 0;
	}

	LRESULT OnWindowMaximize(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		_SendSysCommand(SC_MAXIMIZE);
		return 0;
	}

protected:
	void _SendSysCommand(int wID)
	{
		T* pT = static_cast<T*>(this);
		ATLASSERT(::IsWindow(pT->m_hWnd));
		ATLASSERT(::IsWindow(pT->m_hWndMDIClient));
		BOOL bMaximized = FALSE;
		CWindow wnd = pT->MDIGetActive(&bMaximized);
		if (wnd.m_hWnd == NULL)
			return;

		if (wID == SC_MAXIMIZE && bMaximized)
			return;

		wnd.SendMessage(WM_SYSCOMMAND, (WPARAM)wID);
	}
};

////////////////////////////////////////////////////////////////////////////
// Fixing CMDIChildWindowImpl
// WTL sucks, it support only english menu.

template <class T, int __nWindowMenuPos = 2, class TBase = CMDIWindow, class TWinTraits = CMDIChildWinTraits>
class ATL_NO_VTABLE CMDIChildWindowImplFixed : public CMDIChildWindowImpl<T, TBase, TWinTraits>
{
public:
// Declarations
	typedef CMDIChildWindowImpl<T, TBase, TWinTraits> baseClass;

// Overrides
	static DWORD GetWndStyle(DWORD dwStyle)
	{// created invisibly
		return TWinTraits::GetWndStyle(dwStyle) & ~(WS_VISIBLE | WS_CLIPCHILDREN/* makes drawing slower*/ | WS_CLIPSIBLINGS);
	}

// Message map and handlers
	BEGIN_MSG_MAP(CMDIChildWindowImplFixed)
		MSG_WM_MDIACTIVATE(OnMDIActivate)
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

	void OnMDIActivate(HWND hwndChildDeact, HWND hwndChildAct)
	{
		if(hwndChildAct == m_hWnd && m_hMenu != NULL)
			_SetMDIFrameMenuFixed();
		else if(hwndChildAct == NULL)
			::SendMessage(GetMDIFrame(), WM_MDISETMENU, 0, 0);
		// don't allow CMDIChildWindowImpl to handle this one
		DefWindowProc();
	}

// Methods
	void ActivateFrame(int nCmdShow = -1)
	{// cf.MFC6::CMDIChildWnd::ActivateFrame
		// determine default show command
		if (nCmdShow == -1) {
			// get maximized state of previously active child
			BOOL bMaximized = FALSE;
			HWND hWndActive = MDIGetActive(&bMaximized);
			// convert show command based on current style
			DWORD dwStyle = GetStyle();
			if (bMaximized || !hWndActive || (dwStyle & WS_MAXIMIZE))
				nCmdShow = SW_SHOWMAXIMIZED;
			else if (dwStyle & WS_MINIMIZE)
				nCmdShow = SW_SHOWMINIMIZED;
		}
		// translate default nCmdShow (-1)
		if (nCmdShow == -1) {
			if (!IsWindowVisible())
				nCmdShow = SW_SHOWNORMAL;
			else if (IsIconic())
				nCmdShow = SW_RESTORE;
		}
		// bring to top before showing <-important
		BringToTop(nCmdShow);

		if (nCmdShow != -1) {
			// show the window as specified
			ShowWindow(nCmdShow);
			// and finally, bring to top after showing
			BringToTop(nCmdShow);
		}

		// strangely, MDIRefreshMenu in CMDIWindowFixed::SetMDIFrameMenuFixed ignored
		MDIRefreshMenu();// refresh the Window menu

		ATLTRACE2(atlTraceUser, 4, _T("CChildFrame::ActivateFrame - activated!\n"));
	}

	void BringToTop(int nCmdShow)
	{// cf.MFC6::CFrameWnd::BringToTop
		// place the window on top except for certain nCmdShow
		if (nCmdShow != SW_HIDE &&
			nCmdShow != SW_MINIMIZE && nCmdShow != SW_SHOWMINNOACTIVE &&
			nCmdShow != SW_SHOWNA && nCmdShow != SW_SHOWNOACTIVATE)	{
			// if no last active popup, it will return m_hWnd
			HWND hWndLastPop = ::GetLastActivePopup(m_hWnd);
			::BringWindowToTop(hWndLastPop);
		}
	}

// Implementation
private:
	HMENU _GetStandardWindowMenuFixed(HMENU hMenu)
	{
		int nCount = ::GetMenuItemCount(hMenu);
		if(nCount == -1)
			return NULL;
		int nLen = ::GetMenuString(hMenu, nCount - __nWindowMenuPos, NULL, 0, MF_BYPOSITION);
		if(nLen == 0)
			return NULL;
		LPTSTR lpszText = (LPTSTR)_alloca((nLen + 1) * sizeof(TCHAR));
		if(::GetMenuString(hMenu, nCount - __nWindowMenuPos, lpszText, nLen + 1, MF_BYPOSITION) != nLen)
			return NULL;
//		if(lstrcmp(lpszText, m_lpszWindowMenu))
//			return NULL;
		return ::GetSubMenu(hMenu, nCount - __nWindowMenuPos);
	}

	void _SetMDIFrameMenuFixed()
	{
		ATLTRACE2(atlTraceUser, 4, _T("CMDIWindowFixed::SetMDIFrameMenuFixed\n"));
		HMENU hWindowMenu = _GetStandardWindowMenuFixed(m_hMenu);
		MDISetMenu(m_hMenu, hWindowMenu);
		MDIRefreshMenu();
		::DrawMenuBar(GetMDIFrame());
	}
};


////////////////////////////////////////////////////////////////////////////
// Customizable tool bar handler

template <class T>
class CCustomizableToolBarHandler
{
public:
// Constructor
	CCustomizableToolBarHandler()
		: m_nResourceID(0),
		  m_nItems(0), m_pTBBtn(NULL),
		  m_pUpdateUI(NULL) { }

// Overridable
	void OnTbnCustHelp(HWND hWndToolBar)
	{
		// you can override to deal with HELP.	
	}

// Operations
	void InitCustomizableToolBarHandler(HWND hWndToolBar, UINT nResourceID, CUpdateUIBase* pUpdateUI)
	{
		ATLASSERT(::IsWindow(hWndToolBar));

		m_toolbar = hWndToolBar;
		m_nResourceID = nResourceID;
		m_pUpdateUI = pUpdateUI;

		m_toolbar.ModifyStyle(0, CCS_ADJUSTABLE);
	}

	void InitCustomizableToolBarHandler(HWND hWndToolBar, int* __firstID, int* __lastID, CUpdateUIBase* pUpdateUI)
	{
		if (__firstID == __lastID)
			return;

		for (; __firstID < __lastID; ++__firstID) {
			m_arrCmdID.Add(*__firstID);
		}

		m_toolbar = hWndToolBar;
		m_pUpdateUI = pUpdateUI;

		m_toolbar.ModifyStyle(0, CCS_ADJUSTABLE);
	}
protected:
// Message map and handlers
	BEGIN_MSG_MAP(CCustomizableToolbarHandler)
		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_BEGINADJUST, OnTbnBeginAdjust)
		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_ENDADJUST, OnTbnEndAdjust)
		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_QUERYINSERT, OnTbnQueryInsert)
		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_QUERYDELETE, OnTbnQueryDelete)
		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_GETBUTTONINFO, OnTbnGetButtonInfo)
		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_TOOLBARCHANGE, OnTbnToolBarChange)
		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_RESET, OnTbnReset)
		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_CUSTHELP, OnTbnCustHelp)
//		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_BEGINDRAG, OnTbnBeginDrag)
//		NOTIFY_HANDLER_EX(m_toolbar.GetDlgCtrlID(), TBN_ENDDRAG, OnTbnEndDrag)		
	END_MSG_MAP()

	LRESULT OnTbnBeginAdjust(LPNMHDR pnmh)
	{
		LoadTBBtns();//_LoadTBBtns();
		return 0;
	}

	LRESULT OnTbnEndAdjust(LPNMHDR pnmh)
	{
		ATLTRACE2(atlTraceUser, 4, _T("CCustomizableToolBarHandler::OnTbnEndAdjust\n"));
		_UnloadTBBtns();
		_RefreshBandInfo();
		_UIUpdateToolBarFixed();
		return 0;
	}

	LRESULT OnTbnQueryInsert(LPNMHDR pnmh)
	{
		return TRUE;
	}

	LRESULT OnTbnQueryDelete(LPNMHDR pnmh)
	{
		return TRUE;
	}

	LRESULT OnTbnGetButtonInfo(LPNMHDR pnmh)
	{
		ATLASSERT(m_pTBBtn != NULL);

		LPTBNOTIFY ptbn = (LPTBNOTIFY)pnmh;
		if (ptbn->iItem < m_nItems) {
			CString strText = _GetToolTipText(m_pTBBtn[ptbn->iItem].idCommand);
			if (!strText.IsEmpty())
				::lstrcpy(ptbn->pszText, strText);
			::memcpy(&ptbn->tbButton, &m_pTBBtn[ptbn->iItem], sizeof(TBBUTTON));
			return TRUE;
		}
		else
			return FALSE;
	}

	LRESULT OnTbnToolBarChange(LPNMHDR pnmh)
	{
		ATLTRACE2(atlTraceUser, 4, _T("CCustomizableToolBarHandler::OnTbnToolBarChange\n"));
		_RefreshBandInfo();
		return 0;
	}

	LRESULT OnTbnReset(LPNMHDR pnmh)
	{
		ATLASSERT(m_pTBBtn != NULL);

		int iBtn = m_toolbar.GetButtonCount();
		while (iBtn != 0)
			m_toolbar.DeleteButton(--iBtn);

		m_toolbar.AddButtons(m_nItems, m_pTBBtn);
		return 0;
	}

	LRESULT OnTbnCustHelp(LPNMHDR pnmh)
	{
		T* pT = static_cast<T*>(this);
		pT->OnTbnCustHelp(m_toolbar.m_hWnd);
		return 0;
	}

	LRESULT OnTbnBeginDrag(LPNMHDR pnmh)
	{
		LPTBNOTIFY ptbn = (LPTBNOTIFY)pnmh;
		return 0;
	}

	LRESULT OnTbnEndDrag(LPNMHDR pnmh)
	{
		LPTBNOTIFY ptbn = (LPTBNOTIFY)pnmh;
		return 0;
	}

// Declarations
	struct _AtlToolBarData
	{
		WORD wVersion;
		WORD wWidth;
		WORD wHeight;
		WORD wItemCount;
		//WORD aItems[wItemCount]

		WORD* items()
			{ return (WORD*)(this+1); }
	};

// Data members
	CToolBarCtrl m_toolbar;
	UINT m_nResourceID;
	int m_nItems;
	TBBUTTON* m_pTBBtn;
	CUpdateUIBase* m_pUpdateUI;

	CSimpleArray<int> m_arrCmdID;

// Implementation helpers
	bool LoadTBBtns()
	{
		if (m_nResourceID != 0)
			_LoadTBBtns();
		else
			_LoadTBBtnsMtl();

		return true;
	}

	bool _LoadTBBtnsMtl()
	{
		if (m_arrCmdID.GetSize() == 0)
			return false;

		m_nItems = m_arrCmdID.GetSize();
		m_pTBBtn = new TBBUTTON[m_nItems];
		int nBmp = 0;
		int j = 0;
		for (int i = 0; i < m_nItems; ++i)
		{
			ATLASSERT(j < m_nItems);
			if(m_arrCmdID[i] != 0) {
				m_pTBBtn[j].iBitmap = nBmp++;
				m_pTBBtn[j].idCommand = m_arrCmdID[i];
				m_pTBBtn[j].fsState = TBSTATE_ENABLED;
				m_pTBBtn[j].fsStyle = TBSTYLE_BUTTON;
				m_pTBBtn[j].dwData = 0;
				m_pTBBtn[j].iString = 0;
			}
			else {
				m_pTBBtn[j].iBitmap = 8;
				m_pTBBtn[j].idCommand = 0;
				m_pTBBtn[j].fsState = 0;
				m_pTBBtn[j].fsStyle = TBSTYLE_SEP;
				m_pTBBtn[j].dwData = 0;
				m_pTBBtn[j].iString = 0;
			}
			++j;
		}

		return true;
	}

	bool _LoadTBBtns()
	{
		ATLASSERT(m_pTBBtn == NULL);

		HINSTANCE hInst = _Module.GetResourceInstance();
		HRSRC hRsrc = ::FindResource(hInst, MAKEINTRESOURCE(m_nResourceID), RT_TOOLBAR);
		if (hRsrc == NULL)
			return false;

		HGLOBAL hGlobal = ::LoadResource(hInst, hRsrc);
		if (hGlobal == NULL)
			return false;

		_AtlToolBarData* pData = (_AtlToolBarData*)::LockResource(hGlobal);
		if (pData == NULL)
			return false;
		ATLASSERT(pData->wVersion == 1);

		WORD* pItems = pData->items();
		m_nItems = pData->wItemCount;
		m_pTBBtn = new TBBUTTON[m_nItems];

		int nBmp = 0;
		for(int i = 0, j = 0; i < pData->wItemCount; i++, j++)
		{
			if(pItems[i] != 0)
			{
				m_pTBBtn[j].iBitmap = nBmp++;
				m_pTBBtn[j].idCommand = pItems[i];
				m_pTBBtn[j].fsState = TBSTATE_ENABLED;
				m_pTBBtn[j].fsStyle = TBSTYLE_BUTTON;
				m_pTBBtn[j].dwData = 0;
				m_pTBBtn[j].iString = 0;
			}
			else
			{
				m_pTBBtn[j].iBitmap = 8;
				m_pTBBtn[j].idCommand = 0;
				m_pTBBtn[j].fsState = 0;
				m_pTBBtn[j].fsStyle = TBSTYLE_SEP;
				m_pTBBtn[j].dwData = 0;
				m_pTBBtn[j].iString = 0;
			}
		}

		return true;
	}

	void _UnloadTBBtns()
	{
		m_nItems = 0;
		delete [] m_pTBBtn;
		m_pTBBtn = NULL;
	}

	void _RefreshBandInfo()
	{
		HWND hWndReBar = m_toolbar.GetParent();
		ATLASSERT(::IsWindow(hWndReBar));

		if (!MtlIsKindOf(hWndReBar, REBARCLASSNAME))
			return;
			
		CReBarCtrl rebar(hWndReBar);
		REBARBANDINFO rbBand;
		rbBand.cbSize = sizeof(REBARBANDINFO);
		rbBand.fMask = RBBIM_IDEALSIZE;

		// Calculate the size of the band
		int nBtnCount = m_toolbar.GetButtonCount();
		if(nBtnCount > 0)
		{
			RECT rcTmp;
			BOOL bRet = (BOOL)m_toolbar.GetItemRect(nBtnCount-1, &rcTmp);
			ATLASSERT(bRet);
			rbBand.cxIdeal = rcTmp.right;
			int nIndex = rebar.IdToIndex(m_toolbar.GetDlgCtrlID());
			rebar.SetBandInfo(nIndex, &rbBand);
		}
	}

	BOOL _UIUpdateToolBarFixed()
	{
		if (m_pUpdateUI == NULL)
			return FALSE;
			
		const CUpdateUIBase::_AtlUpdateUIMap* pMap = m_pUpdateUI->m_pUIMap;
		CUpdateUIBase::_AtlUpdateUIData* pUIData = m_pUpdateUI->m_pUIData;
		if(pUIData == NULL)
			return FALSE;

		while(pMap->m_nID != (WORD)-1)
		{
			for(int i = 0; i < m_pUpdateUI->m_UIElements.GetSize(); i++)
			{
				if(m_pUpdateUI->m_UIElements[i].m_wType == CUpdateUIBase::UPDUI_TOOLBAR)
				{
					if(pMap->m_wType & CUpdateUIBase::UPDUI_TOOLBAR)
						m_pUpdateUI->UIUpdateToolBarElement(pMap->m_nID, pUIData, m_pUpdateUI->m_UIElements[i].m_hWnd);
				}
			}
			pMap++;
			pUIData->m_wState &= ~CUpdateUIBase::UPDUI_TOOLBAR;
			pUIData++;
		}

		m_pUpdateUI->m_wDirtyType &= ~CUpdateUIBase::UPDUI_TOOLBAR;
		return TRUE;
	}

	CString _GetToolTipText(int idCtrl)
	{
		if(idCtrl != 0)
		{
			TCHAR szBuff[256];
			szBuff[0] = 0;
			int nRet = ::LoadString(_Module.GetResourceInstance(), idCtrl, szBuff, 256);
			for (int i = 0; i < nRet; i++)
			{
				if (szBuff[i] == _T('\n'))
				{
					return CString(&szBuff[i + 1]);
				}
			}
		}

		return CString();
	}
};

template <class T>
class CClipboardViewer
{
public:
// Data members
	HWND m_hWndNext;
	bool m_bFirst;
	bool m_bInitialUpdate;
	bool m_bSetClipboardViewer;

// Constructor
	CClipboardViewer()
		: m_hWndNext(NULL), m_bFirst(true), m_bInitialUpdate(false),
		  m_bSetClipboardViewer(false)
	{
	}

	~CClipboardViewer()
	{
		ATLASSERT(m_bSetClipboardViewer == false);
	}

	void InstallClipboardViewer(bool bInitialUpdate = false)
	{
		if (m_bSetClipboardViewer)
			return;

		T* pT = static_cast<T*>(this);
		m_bInitialUpdate = bInitialUpdate;
		m_hWndNext = ::SetClipboardViewer(pT->m_hWnd);
		m_bInitialUpdate = bInitialUpdate;

		m_bSetClipboardViewer = true;
	}

	void UninstallClipboardViewer()
	{
		if (!m_bSetClipboardViewer)
			return;

		T* pT = static_cast<T*>(this);
		::ChangeClipboardChain(pT->m_hWnd, m_hWndNext);
		m_hWndNext			= NULL;
		m_bFirst			= true;
		m_bInitialUpdate	= false;

		m_bSetClipboardViewer = false;
	}

// Overridables
	void OnUpdateClipboard()
	{
		ATLASSERT(FALSE);
	}

// Message map and handlers
	BEGIN_MSG_MAP(CClipboardViewer)
		MSG_WM_DRAWCLIPBOARD(OnDrawClipboard)
		MSG_WM_CHANGECBCHAIN(OnChangeCBChain)
		MSG_WM_DESTROY(OnDestroy)
	END_MSG_MAP()

	void OnDrawClipboard()
	{
		ATLTRACE2(atlTraceUser, 4, _T("CClipboardViewer::OnDrawClipboard\n"));
		T* pT = static_cast<T*>(this);
		
		if (m_hWndNext != NULL)
			::PostMessage(m_hWndNext, WM_DRAWCLIPBOARD, 0, 0);//!!Post!?
		
		if (m_bFirst) {				// first time
			if (m_bInitialUpdate) {	// callback
				pT->OnUpdateClipboard();
				m_bInitialUpdate = false;
			}
			m_bFirst = false;
		}
		else {
			pT->OnUpdateClipboard();
		}
	}

	void OnChangeCBChain(HWND hWndRemove, HWND hWndAfter)
	{
		ATLTRACE2(atlTraceUser, 4, _T("CClipboardViewer::OnChangeCBChain\n"));
		if (m_hWndNext == hWndRemove)
			m_hWndNext = hWndAfter;
		else if (m_hWndNext != NULL)// SendMessage if I'm not last
			::SendMessage(m_hWndNext, WM_CHANGECBCHAIN, (WPARAM)hWndRemove, (LPARAM)hWndAfter);
	}

	void OnDestroy()
	{
		SetMsgHandled(FALSE);
		UninstallClipboardViewer();
	}
};

inline void MtlDestroyImageLists(HWND hWndToolBar)
{
	ATLTRACE2(atlTraceUser, 4, _T("MtlDestroyImageList\n"));
	CToolBarCtrl toolbar(hWndToolBar);
	HIMAGELIST hImg = toolbar.GetImageList();
	if (hImg) {
		ATLTRACE2(atlTraceUser, 4, _T(" delete imagelist\n"));
		MTLVERIFY(ImageList_Destroy(hImg));
	}

	hImg = toolbar.GetHotImageList();
	if (hImg) {
		ATLTRACE2(atlTraceUser, 4, _T(" delete hot imagelist\n"));
		MTLVERIFY(ImageList_Destroy(hImg));
	}
}

// __nID == 0 then separator
HWND MtlCreateSimpleToolBarCtrl(HWND hWndParent, int* __firstID, int* __lastID,
	UINT nImageBmpID, UINT nHotImageBmpID, int cx, int cy, COLORREF clrMask, UINT nFlags = ILC_COLOR8,
	DWORD dwStyle = ATL_SIMPLE_TOOLBAR_PANE_STYLE, UINT nID = ATL_IDW_TOOLBAR)
{
		int nCount = __lastID - __firstID;
		if (nCount == 0)
			return NULL;

		CImageList imgs;
		MTLVERIFY(imgs.Create(cx, cy, nFlags | ILC_MASK, nCount, 1));

		CBitmap bmp;
		MTLVERIFY(bmp.LoadBitmap(nImageBmpID));
		imgs.Add(bmp, clrMask);


		CImageList imgsHot;
		MTLVERIFY(imgsHot.Create(cx, cy, nFlags | ILC_MASK, nCount, 1));

		CBitmap bmpHot;
		MTLVERIFY(bmpHot.LoadBitmap(nHotImageBmpID));
		imgsHot.Add(bmpHot, clrMask);


		TBBUTTON* pTBBtn = (TBBUTTON*)_alloca(nCount * sizeof(TBBUTTON));

		int nBmp = 0;
		int j = 0;
		for (; __firstID < __lastID; ++__firstID) {
			ATLASSERT(j < nCount);
			if(*__firstID != 0) {
				pTBBtn[j].iBitmap = nBmp++;
				pTBBtn[j].idCommand = *__firstID;
				pTBBtn[j].fsState = TBSTATE_ENABLED;
				pTBBtn[j].fsStyle = TBSTYLE_BUTTON;
				pTBBtn[j].dwData = 0;
				pTBBtn[j].iString = 0;
			}
			else {
				pTBBtn[j].iBitmap = 8;
				pTBBtn[j].idCommand = 0;
				pTBBtn[j].fsState = 0;
				pTBBtn[j].fsStyle = TBSTYLE_SEP;
				pTBBtn[j].dwData = 0;
				pTBBtn[j].iString = 0;
			}
			// using toolbar dropdown menu , must set TBSTYLE_DROPDOWN.
			if((*__firstID == ID_VIEW_BACK) || (*__firstID == ID_VIEW_FORWARD) || (*__firstID == ID_VIEW_FAVEXPBAR)) // UDT DGSTR
					pTBBtn[j].fsStyle |= TBSTYLE_DROPDOWN;
			++j;
		}

		CToolBarCtrl toolbar = ::CreateWindowEx(0, TOOLBARCLASSNAME, NULL, dwStyle, 0,0,100,100,
				hWndParent, (HMENU)LongToHandle(nID), _Module.GetModuleInstance(), NULL);

		toolbar.SetButtonStructSize(sizeof(TBBUTTON));
		toolbar.AddButtons(nCount, pTBBtn);
		toolbar.SetImageList(imgs);
		toolbar.SetHotImageList(imgsHot);

		return toolbar.m_hWnd;
	}

#ifdef __ATLCTRLW_H__
BOOL MtlCmdBarLoadImages(CCommandBarCtrl& cmdbar, int* __firstID, int* __lastID,
	UINT nImageBmpID, int cx, int cy, COLORREF clrMask, UINT nFlags = ILC_COLOR8)
{
	// Add bitmap to our image list (create it if it doesn't exist)
	int nCount = __lastID - __firstID;
	if (nCount == 0)
		return FALSE;

	if(cmdbar.m_hImageList == NULL)
	{
		cmdbar.m_hImageList = ::ImageList_Create(cx, cy, nFlags | ILC_MASK, nCount, 1);
		ATLASSERT(cmdbar.m_hImageList != NULL);
		if(cmdbar.m_hImageList == NULL)
			return FALSE;
	}

	CBitmap bmp;
	bmp.LoadBitmap(nImageBmpID);
	ATLASSERT(bmp.m_hBitmap != NULL);
	if(bmp.m_hBitmap == NULL)
		return FALSE;
	if(::ImageList_AddMasked(cmdbar.m_hImageList, bmp, cmdbar.m_clrMask = clrMask) == -1)
		return FALSE;

	// Fill the array with command IDs
	for (; __firstID < __lastID; ++__firstID) {
		if(*__firstID != 0)
			cmdbar.m_arrCommand.Add(*__firstID);
	}

	ATLASSERT(::ImageList_GetImageCount(cmdbar.m_hImageList) == cmdbar.m_arrCommand.GetSize());
	if(::ImageList_GetImageCount(cmdbar.m_hImageList) != cmdbar.m_arrCommand.GetSize())
		return FALSE;

	// Set some internal stuff
	cmdbar.m_szBitmap.cx = cx;
	cmdbar.m_szBitmap.cy = cy;
	cmdbar.m_szButton.cx = cmdbar.m_szBitmap.cx + 2 * CCommandBarCtrl::s_kcxButtonMargin;
	cmdbar.m_szButton.cy = cmdbar.m_szBitmap.cy + 2 * CCommandBarCtrl::s_kcyButtonMargin;

	return TRUE;
}
#endif

////////////////////////////////////////////////////////////////////////////
// Mouse Wheel Support for Windows 95 or Windows NT 3.51

class CMSMouseWheelMessageHandlerBase
{
public:
	DECLARE_REGISTERED_MESSAGE(MSWHEEL_ROLLMSG)
};

template <class T>
class CMSMouseWheelMessageHandler : public CMSMouseWheelMessageHandlerBase
{
public:
	BEGIN_MSG_MAP(CMSMouseWheelMessageHandler<T>)
		MESSAGE_HANDLER(GET_REGISTERED_MESSAGE(MSWHEEL_ROLLMSG), OnRegisteredMouseWheel)
	END_MSG_MAP()

	LRESULT OnRegisteredMouseWheel(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{// pasted from MFC6
		// convert from MSH_MOUSEWHEEL to WM_MOUSEWHEEL

		WORD keyState = 0;
		keyState |= (::GetKeyState(VK_CONTROL) < 0) ? MK_CONTROL : 0;
		keyState |= (::GetKeyState(VK_SHIFT) < 0) ? MK_SHIFT : 0;

		LRESULT lResult;
		HWND hwFocus = ::GetFocus();
		const HWND hwDesktop = ::GetDesktopWindow();

		if (hwFocus == NULL)
		{
			T* pT = static_cast<T*>(this);
			lResult = pT->SendMessage(WM_MOUSEWHEEL, (wParam << 16) | keyState, lParam);
		}
		else
		{
			do {
				lResult = ::SendMessage(hwFocus, WM_MOUSEWHEEL,
					(wParam << 16) | keyState, lParam);
				hwFocus = ::GetParent(hwFocus);
			}
			while (lResult == 0 && hwFocus != NULL && hwFocus != hwDesktop);
		}
		return lResult;
	}
};

////////////////////////////////////////////////////////////////////////////
// MDI main frame Title changer
//  but I've found there is no easy way to change task bar's text if MDI...

template <class T>
class CMDIFrameTitleUpsideDownMessageHandler
{
public:
// Data members
	bool m_bValid;

// Constructor
	CMDIFrameTitleUpsideDownMessageHandler() : m_bValid(true) { }

// Methods
	void UpdateMDIFrameTitle()// no effect
	{
		T* pT = static_cast<T*>(this);
		BOOL bMaximized = FALSE;
		HWND hWndActive = pT->MDIGetActive(&bMaximized);
		if (hWndActive == NULL || bMaximized == FALSE) {
			return;
		}

		MtlSetWindowText(pT->m_hWnd, _Make_UpsideDownTitle(hWndActive));
	}

// Message map and handlers
	BEGIN_MSG_MAP(CMDIFrameTitleUpsideDownMessageHandler)
//		MESSAGE_HANDLER(WM_GETTEXTLENGTH, OnGetTextLength) no need, cuz original buffer size is always larger.
//		MESSAGE_HANDLER(WM_SETTEXT, OnSetText) never called by MDI system
		MESSAGE_HANDLER(WM_GETTEXT, OnGetText)
	END_MSG_MAP()

	LRESULT OnGetText(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		UINT cchTextMax = (UINT)wParam;   // number of characters to copy 
		LPTSTR lpszText = (LPTSTR)lParam;
		bHandled = FALSE;

		if (!m_bValid)
			return 0;

		T* pT = static_cast<T*>(this);
		// first, find the mdi child's title
		BOOL bMaximized = FALSE;
		HWND hWndActive = pT->MDIGetActive(&bMaximized);
		if (hWndActive == NULL || bMaximized == FALSE)
			return 0;		

		CString strApp; strApp.LoadString(IDR_MAINFRAME);
		CString strTitle = MtlGetWindowText(hWndActive) + _T(" - ") + strApp;
		::lstrcpyn(lpszText, strTitle, cchTextMax);	

		bHandled = TRUE;
		return min(strTitle.GetLength(), cchTextMax);
	}

	CString _Make_UpsideDownTitle(HWND hWndMDIChild)
	{
		CString strChild = MtlGetWindowText(hWndMDIChild);
		CString strApp; strApp.LoadString(IDR_MAINFRAME);
		return strChild + _T(" - ") + strApp;
	}

	LRESULT OnSetText(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		CString strText = (LPCTSTR)lParam;

		// first, find the mdi child's title
		int nFirstIndex = strText.Find(_T('['));
		int nLastIndex = strText.ReverseFind(_T(']'));

		if (nFirstIndex > nLastIndex || nFirstIndex == -1 || nFirstIndex == -1) {
			bHandled = FALSE;
			return FALSE;
		}

		CString strChild = strText.Mid(nFirstIndex, nLastIndex - nFirstIndex + 1);

		// second, find the application's title
		CString strApp; strApp.LoadString(IDR_MAINFRAME);
		strText = strChild + _T(" - ") + strApp;
		
 
		T* pT = static_cast<T*>(this);
		return pT->DefWindowProc(uMsg, wParam, (LPARAM)(LPCTSTR)strText);
	}
};

template <class T>
class CHelpMessageLine
{
public:
	BEGIN_MSG_MAP(CHelpMessageLine)
		MESSAGE_HANDLER(WM_MENUSELECT, OnMenuSelect)
	END_MSG_MAP()

	LRESULT OnMenuSelect(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		T* pT = static_cast<T*>(this);

		if(pT->m_hWndStatusBar == NULL)
			return pT->DefWindowProc(uMsg, wParam, lParam);

		HMENU hMenu = (HMENU)lParam;
		WORD wFlags = HIWORD(wParam);
		UINT uItem = (UINT)LOWORD(wParam);   // menu item or submenu index 

		CString strHelp;
		if(wFlags == 0xFFFF && hMenu == NULL) {	// menu closing
			::SendMessage(pT->m_hWndStatusBar, SB_SIMPLE, FALSE, 0L);
		}
		else {
			if(!(wFlags & MF_POPUP)) {
				WORD wID = LOWORD(wParam);
				// check for special cases
				if(wID >= 0xF000 && wID < 0xF1F0)				// system menu IDs
					wID = (WORD)(((wID - 0xF000) >> 4) + ATL_IDS_SCFIRST);
				else if(wID >= ID_FILE_MRU_FIRST && wID <= ID_FILE_MRU_LAST)	// MRU items
					wID = ATL_IDS_MRU_FILE;
				else if(wID >= ATL_IDM_FIRST_MDICHILD)				// MDI child windows
					wID = ATL_IDS_MDICHILD;

				strHelp.LoadString(wID);
				int nStart = strHelp.Find(_T('\t'));
				int nLast = strHelp.Find(_T('\n'));

				if (nStart == -1 && nLast != -1) {
					strHelp = strHelp.Mid(0, nLast);
				}
				else if (nStart != -1 && nLast == -1) {
					strHelp = strHelp.Right(strHelp.GetLength() - nStart - 1);
				}
				else if (nStart != -1 && nLast != -1) {
					ATLASSERT(nStart < nLast);
					strHelp = strHelp.Mid(nStart + 1, nLast - nStart - 1);
				}
			}
			else // popup support
			{
				HMENU hMenuPopup = ::GetSubMenu(hMenu, uItem);
				
				UINT nID = -1;
				TCHAR szString[100];

				// search the first non-popup menu's ID
				for (int i = 0; i < ::GetMenuItemCount(hMenuPopup); ++i) {
					// Fixed by Nisizawa@mcomp.co.jp, thanks.
					// Note. GetMenuItemInfo often change mii.dwTypeData.
					//       even if the API reference says not.
					CMenuItemInfo mii;
					mii.fMask = MIIM_ID | MIIM_TYPE | MIIM_SUBMENU;
					mii.dwTypeData = szString;
					mii.cch = 100;
					
					if (!::GetMenuItemInfo(hMenuPopup, i, TRUE, &mii))
						continue;

					bool bSkip = _check_flag(MFT_SEPARATOR, mii.fType) || mii.hSubMenu != NULL;
					nID =  bSkip ? -1 : mii.wID;
					if (nID != -1) {
						if (nID >= ID_FILE_MRU_FIRST && nID <= ID_FILE_MRU_LAST)	// MRU items
							nID = ATL_IDS_MRU_FILE;
						else if (nID >= ATL_IDM_FIRST_MDICHILD)				// MDI child windows
							nID = ATL_IDS_MDICHILD;
						break;
					}
				}

				if (nID != -1) {
					strHelp.LoadString(nID);
					int nIndex = strHelp.Find(_T('\t'));
					if (nIndex != -1) {
						strHelp = strHelp.Left(nIndex);
					}
					else {
						strHelp.Empty();
					}
				}
			}

			::SendMessage(pT->m_hWndStatusBar, SB_SIMPLE, TRUE, 0L);
			::SendMessage(pT->m_hWndStatusBar, SB_SETTEXT, (255 | SBT_NOBORDERS), (LPARAM)(LPCTSTR)strHelp);
		}

		return pT->DefWindowProc(uMsg, wParam, lParam);
	}
};

/////////////////////////////////////////////////////////////////////////////
// CInitDialogImpl - property page that supports DropDownList initialization

#ifndef _ATL_RT_DLGINIT
	#define _ATL_RT_DLGINIT  MAKEINTRESOURCE(240)
#endif

template <class T, class TBase = CWindow>
class ATL_NO_VTABLE CInitDialogImpl : public CDialogImpl< T, TBase >
{
public:
	typedef CInitDialogImpl< T, TBase > thisClass;
	typedef CDialogImpl< T, TBase > baseClass;

// Data members
	HGLOBAL m_hInitData;

// Constructor/destructor
	CInitDialogImpl() :	m_hInitData(NULL)
	{
		T* pT = static_cast<T*>(this);
		pT;	// avoid level 4 warning

		HINSTANCE hInstance = _Module.GetResourceInstance();
		LPCTSTR lpTemplateName = MAKEINTRESOURCE(pT->IDD);
		HRSRC hDlg = ::FindResource(hInstance, lpTemplateName, (LPTSTR)RT_DIALOG);
		if(hDlg != NULL)
		{
			HRSRC hDlgInit = ::FindResource(hInstance, lpTemplateName, (LPTSTR)_ATL_RT_DLGINIT);
			if(hDlgInit != NULL)
			{
				m_hInitData = ::LoadResource(hInstance, hDlgInit);
			}
		}
		else
		{
			ATLASSERT(FALSE && _T("CInitDialogImpl - Cannot find dialog template!"));
		}
	}

	~CInitDialogImpl()
	{
		if(m_hInitData != NULL)
		{
			UnlockResource(m_hInitData);
			::FreeResource(m_hInitData);
		}
	}
// Methods
	bool ExecuteDlgInit()
	{
		BYTE* pInitData = (BYTE*)::LockResource(m_hInitData);
		return _ExecuteDlgInit(pInitData);
	}

	bool _ExecuteDlgInit(LPVOID lpResource)
	{// cf.MFC
		bool bSuccess = true;
		if (lpResource != NULL)
		{
			UNALIGNED WORD* lpnRes = (WORD*)lpResource;
			while (bSuccess && *lpnRes != 0)
			{
				WORD nIDC = *lpnRes++;
				WORD nMsg = *lpnRes++;
				DWORD dwLen = *((UNALIGNED DWORD*&)lpnRes)++;

				// In Win32 the WM_ messages have changed.  They have
				// to be translated from the 32-bit values to 16-bit
				// values here.

				#define WIN16_LB_ADDSTRING  0x0401
				#define WIN16_CB_ADDSTRING  0x0403
				#define AFX_CB_ADDSTRING    0x1234

				// unfortunately, WIN16_CB_ADDSTRING == CBEM_INSERTITEM
				if (nMsg == AFX_CB_ADDSTRING)
					nMsg = CBEM_INSERTITEM;
				else if (nMsg == WIN16_LB_ADDSTRING)
					nMsg = LB_ADDSTRING;
				else if (nMsg == WIN16_CB_ADDSTRING)
					nMsg = CB_ADDSTRING;

			// check for invalid/unknown message types
				ATLASSERT(nMsg == LB_ADDSTRING || nMsg == CB_ADDSTRING ||
					nMsg == CBEM_INSERTITEM);

#ifdef _DEBUG
				// For AddStrings, the count must exactly delimit the
				// string, including the NULL termination.  This check
				// will not catch all mal-formed ADDSTRINGs, but will
				// catch some.
				if (nMsg == LB_ADDSTRING || nMsg == CB_ADDSTRING || nMsg == CBEM_INSERTITEM)
					ATLASSERT(*((LPBYTE)lpnRes + (UINT)dwLen - 1) == 0);
#endif

				if (nMsg == CBEM_INSERTITEM)
				{
					USES_CONVERSION;
					COMBOBOXEXITEM item;
					item.mask = CBEIF_TEXT;
					item.iItem = -1;
					item.pszText = A2T(LPSTR(lpnRes));	

					if (::SendDlgItemMessage(m_hWnd, nIDC, nMsg, 0, (LPARAM) &item) == -1)
						bSuccess = false;
				}
				else if (nMsg == LB_ADDSTRING || nMsg == CB_ADDSTRING)
				{
					// List/Combobox returns -1 for error
					if (::SendDlgItemMessageA(m_hWnd, nIDC, nMsg, 0, (LPARAM) lpnRes) == -1)
						bSuccess = false;
				}


				// skip past data
				lpnRes = (WORD*)((LPBYTE)lpnRes + (UINT)dwLen);
			}
		}

		// send update message to all controls after all other siblings loaded
//		if (bSuccess)
//			SendMessageToDescendants(WM_INITIALUPDATE, 0, 0, FALSE, FALSE);

		return bSuccess;
	}

// DDX methods
	void DDX_CBIndex(UINT nID, int& nValue, BOOL bSave)
	{
		HWND hWndCtrl = GetDlgItem(nID);
		ATLASSERT(::IsWindow(hWndCtrl));
		if(bSave)
		{
			nValue = (int)::SendMessage(hWndCtrl, CB_GETCURSEL, 0, 0L);
			ATLASSERT(nValue >= 0 && nValue < (int)::SendMessage(hWndCtrl, CB_GETCOUNT, 0, 0L));
		}
		else
		{
			if(nValue < 0 || nValue >= (int)::SendMessage(hWndCtrl, CB_GETCOUNT, 0, 0L))
			{
				ATLTRACE2(atlTraceUI, 0, "ATL: Warning - dialog data combobox value (%d) out of range.\n", nValue);
				nValue = 0;  // default to off
			}
			int nRet = ::SendMessage(hWndCtrl, CB_SETCURSEL, nValue, 0L);
			ATLASSERT(nRet != CB_ERR);
		}
	}
};

/////////////////////////////////////////////////////////////////////////////
// CInitPropertyPageImpl - property page that supports DropDownList initialization
//   That is, the subset of CAxProtertyPageImpl
//   If you use combobox ex, don't miss ICC_USEREX_CLASSES.

#define DDX_CBINDEX(nID, var) \
		if(nCtlID == (UINT)-1 || nCtlID == nID) \
			DDX_CBIndex(nID, var, bSaveAndValidate);

template <class T, class TBase = CPropertyPageWindow>
class ATL_NO_VTABLE CInitPropertyPageImpl : public CPropertyPageImpl< T, TBase >
{
public:
	typedef CInitPropertyPageImpl< T, TBase > thisClass;
	typedef CPropertyPageImpl< T, TBase > baseClass;

// Data members
	HGLOBAL m_hInitData;

// Constructor/destructor
	CInitPropertyPageImpl(_U_STRINGorID title = (LPCTSTR)NULL) : 
			CPropertyPageImpl< T, TBase >(title),
			m_hInitData(NULL)
	{
		T* pT = static_cast<T*>(this);
		pT;	// avoid level 4 warning

		HINSTANCE hInstance = _Module.GetResourceInstance();
		LPCTSTR lpTemplateName = MAKEINTRESOURCE(pT->IDD);
		HRSRC hDlg = ::FindResource(hInstance, lpTemplateName, (LPTSTR)RT_DIALOG);
		if(hDlg != NULL)
		{
			HRSRC hDlgInit = ::FindResource(hInstance, lpTemplateName, (LPTSTR)_ATL_RT_DLGINIT);
			if(hDlgInit != NULL)
			{
				m_hInitData = ::LoadResource(hInstance, hDlgInit);
			}
		}
		else
		{
			ATLASSERT(FALSE && _T("CInitPropertyPageImpl - Cannot find dialog template!"));
		}
	}

	~CInitPropertyPageImpl()
	{
		if(m_hInitData != NULL)
		{
			UnlockResource(m_hInitData);
			::FreeResource(m_hInitData);
		}
	}
	
	BEGIN_MSG_MAP(thisClass)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

	LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		bHandled = FALSE;

		BYTE* pInitData = (BYTE*)::LockResource(m_hInitData);
		ExecuteDlgInit(pInitData);

		return TRUE;
	}

// Methods
	bool ExecuteDlgInit(LPVOID lpResource)
	{// cf.MFC
		bool bSuccess = true;
		if (lpResource != NULL)
		{
			UNALIGNED WORD* lpnRes = (WORD*)lpResource;
			while (bSuccess && *lpnRes != 0)
			{
				WORD nIDC = *lpnRes++;
				WORD nMsg = *lpnRes++;
				DWORD dwLen = *((UNALIGNED DWORD*&)lpnRes)++;

				// In Win32 the WM_ messages have changed.  They have
				// to be translated from the 32-bit values to 16-bit
				// values here.

				#define WIN16_LB_ADDSTRING  0x0401
				#define WIN16_CB_ADDSTRING  0x0403
				#define AFX_CB_ADDSTRING    0x1234

				// unfortunately, WIN16_CB_ADDSTRING == CBEM_INSERTITEM
				if (nMsg == AFX_CB_ADDSTRING)
					nMsg = CBEM_INSERTITEM;
				else if (nMsg == WIN16_LB_ADDSTRING)
					nMsg = LB_ADDSTRING;
				else if (nMsg == WIN16_CB_ADDSTRING)
					nMsg = CB_ADDSTRING;

			// check for invalid/unknown message types
				ATLASSERT(nMsg == LB_ADDSTRING || nMsg == CB_ADDSTRING ||
					nMsg == CBEM_INSERTITEM);

#ifdef _DEBUG
				// For AddStrings, the count must exactly delimit the
				// string, including the NULL termination.  This check
				// will not catch all mal-formed ADDSTRINGs, but will
				// catch some.
				if (nMsg == LB_ADDSTRING || nMsg == CB_ADDSTRING || nMsg == CBEM_INSERTITEM)
					ATLASSERT(*((LPBYTE)lpnRes + (UINT)dwLen - 1) == 0);
#endif

				if (nMsg == CBEM_INSERTITEM)
				{
					USES_CONVERSION;
					COMBOBOXEXITEM item;
					item.mask = CBEIF_TEXT;
					item.iItem = -1;
					item.pszText = A2T(LPSTR(lpnRes));	

					if (::SendDlgItemMessage(m_hWnd, nIDC, nMsg, 0, (LPARAM) &item) == -1)
						bSuccess = false;
				}
				else if (nMsg == LB_ADDSTRING || nMsg == CB_ADDSTRING)
				{
					// List/Combobox returns -1 for error
					if (::SendDlgItemMessageA(m_hWnd, nIDC, nMsg, 0, (LPARAM) lpnRes) == -1)
						bSuccess = false;
				}


				// skip past data
				lpnRes = (WORD*)((LPBYTE)lpnRes + (UINT)dwLen);
			}
		}

		// send update message to all controls after all other siblings loaded
//		if (bSuccess)
//			SendMessageToDescendants(WM_INITIALUPDATE, 0, 0, FALSE, FALSE);

		return bSuccess;
	}

// DDX methods
	void DDX_CBIndex(UINT nID, int& nValue, BOOL bSave)
	{
		HWND hWndCtrl = GetDlgItem(nID);
		if(bSave)
		{
			nValue = (int)::SendMessage(hWndCtrl, CB_GETCURSEL, 0, 0L);
			ATLASSERT(nValue >= 0 && nValue < (int)::SendMessage(hWndCtrl, CB_GETCOUNT, 0, 0L));
		}
		else
		{
			if(nValue < 0 || nValue >= (int)::SendMessage(hWndCtrl, CB_GETCOUNT, 0, 0L))
			{
				ATLTRACE2(atlTraceUI, 0, "ATL: Warning - dialog data combobox value (%d) out of range.\n", nValue);
				nValue = 0;  // default to off
			}
			::SendMessage(hWndCtrl, CB_SETCURSEL, nValue, 0L);
		}
	}
};

/////////////////////////////////////////////////////////////////////////////
// WTL forgot to center a sheet...
class CCenterPropertySheet : public CPropertySheetImpl<CCenterPropertySheet>
{
public:
	bool m_bCentered;

	CCenterPropertySheet(_U_STRINGorID title = (LPCTSTR)NULL, UINT uStartPage = 0, HWND hWndParent = NULL)
		: CPropertySheetImpl<CCenterPropertySheet>(title, uStartPage, hWndParent), m_bCentered(false)
	{ }

	BEGIN_MSG_MAP(CCenterPropertySheet)
		MESSAGE_HANDLER(WM_SIZE, OnSize)
		MESSAGE_HANDLER(WM_COMMAND, CPropertySheetImpl<CCenterPropertySheet>::OnCommand)
	END_MSG_MAP()

	LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		if (!m_bCentered) {
			// now center the property sheet
			CenterWindow(GetParent());
			m_bCentered = true;
		}

		return 0;
	}
};

/////////////////////////////////////////////////////////////////////////////
// menu helper
inline int MtlGetCmdIDFromAccessKey(CMenuHandle menu, const CString& strAccess, bool bFromBottom = true)
{
	int nID = -1;
	CString strItem;

	if (bFromBottom)
	{
		for (int i = menu.GetMenuItemCount() - 1; i >= 0 ; --i)
		{
			if (menu.GetMenuString(i, strItem, MF_BYPOSITION) == 0)
				continue;

			if (strItem.Find(strAccess) != -1) {
				nID = menu.GetMenuItemID(i);
				break;
			}
		}
	}
	else
	{
		for (int i = 0; i < menu.GetMenuItemCount(); ++i)
		{
			if (menu.GetMenuString(i, strItem, MF_BYPOSITION) == 0)
				continue;

			if (strItem.Find(strAccess) != -1) {
				nID = menu.GetMenuItemID(i);
				break;
			}
		}
	}

	return nID;
}


/////////////////////////////////////////////////////////////////////////////
}  //namespace MTL

#endif // __MTLUSER_H__

