// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////
#pragma once

#include "MenuControl.h"
#include "MainFrameFileDropTarget.h"

#ifdef _DEBUG
	const bool _Donut_MainFrame_traceOn = false;
	#define dmfTRACE if (_Donut_MainFrame_traceOn) ATLTRACE
#else
	#define dmfTRACE
#endif

class CMainFrame : public CMDIFrameWindowImpl<CMainFrame>,
		public CMessageFilter, public CIdleHandler,
		public CAppCommandHandler<CMainFrame>,
		public CUpdateCmdUI<CMainFrame>,
		public CWindowMenuCommandHandler<CMainFrame>,
		public CDDEMessageHandler<CMainFrame>,
		public CMDIFrameTitleUpsideDownMessageHandlerWeb<CMainFrame>,
		public CHelpMessageLine<CMainFrame>,
		public CMSMouseWheelMessageHandler<CMainFrame>,
		public CMainFrameFileDropTarget<CMainFrame>,
		public CReBarVerticalBorder<CMainFrame>
{
public:
// Declarelations
	DECLARE_FRAME_WND_CLASS(_T("WTL:Donut"), IDR_MAINFRAME)
	DECLARE_MTL_REBAR_UPDATELAYOUT()
	typedef CMDIFrameWindowImpl<CMainFrame> baseClass;

// Constants
	enum { _nPosFavoriteMenu = 3, _nPosFavGroupMenu = 2, _nPosViewMenu = 2,
		_nPosFavoriteMenuOfTab = 2, _nPosMRU = 12/*11*/, _nPosWindowMenu = 5, _nWindowMenuItemCount = 14 }; // UDT DGSTR

// Daba members
	CCommandBarCtrl2 m_CmdBar;
	CDonutToolBar m_ToolBar;
	CDonutLinksBarCtrl<CMainFrame> m_LinkBar;
	CDonutAddressBar m_AddressBar;
	CMDITabCtrl	m_MDITab;
	CMultiPaneStatusBarCtrl m_wndStatusBar;
	CProgressBarCtrl m_wndProgress;

	CMainOption m_MainOption;
//	CDonutSecurityZone m_secZone;

	CSplitterWindow m_wndSplit;
	CDonutExplorerBar m_ExplorerBar;

	CDonutFavoritesMenu<CMainFrame> m_FavoriteMenu;
	CFavoriteGroupMenu<CMainFrame> m_FavGroupMenu;

	CMenuControl m_mcCmdBar;
	CMenuControl m_mcToolBar;

	CContainedWindow m_wndMDIClient;

	// FullScreen support
	BOOL m_bStatusBarVisibleUnlessFullScreen;

	UINT m_nBackUpTimerID;
	CMenuHandle m_menuWindow;

	HWND m_hWndFocus;

	// Constructor/Destructor
	CMainFrame() :
		m_FavoriteMenu(this, ID_INSERTPOINT_FAVORITEMENU),
		m_FavGroupMenu(this, ID_INSERTPOINT_GROUPMENU),
		m_LinkBar(this, ID_INSERTPOINT_FAVORITEMENU),
		m_nBackUpTimerID(0), /*m_FavTree(this), */m_wndMDIClient(this, 1),
		CDDEMessageHandler<CMainFrame>(_T("Donut")), m_hWndFocus(NULL),
		m_ExplorerBar(m_wndSplit)
	{
	}

	~CMainFrame()
	{
		if (m_wndMDIClient.IsWindow())
			m_wndMDIClient.UnsubclassWindow();
	}

// Overrides
	virtual BOOL PreTranslateMessage(MSG* pMsg)
	{
		if (m_CmdBar.PreTranslateMessage(pMsg))
			return TRUE;
		
		BYTE ptFlag = m_AddressBar.PreTranslateMessage(pMsg);
		if (ptFlag == _MTL_TRANSLATE_HANDLE)
			return TRUE;
		else if (ptFlag == _MTL_TRANSLATE_WANT)
			return FALSE;

		ptFlag = m_ExplorerBar.PreTranslateMessage(pMsg);
		if (ptFlag == _MTL_TRANSLATE_HANDLE)
			return TRUE;
		else if (ptFlag == _MTL_TRANSLATE_WANT)
			return FALSE;

		if (baseClass::PreTranslateMessage(pMsg))
			return TRUE;

		HWND hWnd = MDIGetActive();
		if (hWnd != NULL && (BOOL)::SendMessage(hWnd, WM_FORWARDMSG, 0, (LPARAM)pMsg))
			return TRUE;
		
		return FALSE;// IsDialogMessage(pMsg);
	}

	virtual BOOL OnIdle()
	{
		// Note. under 0.01 sec (in dbg-mode on 330mhz cpu)
		CmdUIUpdateToolBars();
		CmdUIUpdateStatusBar(m_hWndStatusBar, ID_DEFAULT_PANE);
		CmdUIUpdateChildWindow(m_hWndStatusBar, IDC_PROGRESS);
		m_mcCmdBar.UpdateMDIMenuControl();
		_FocusChecker();

		if (_check_flag(MAIN_EX_KILLDIALOG, CMainOption::s_dwMainExtendedStyle))
			CDialogKiller2::KillDialog();

		return FALSE;
	}

	void _OnIdleCritical()
	{
		if (MtlIsApplicationActive(m_hWnd)) {
			CmdUIUpdateToolBars();
			CmdUIUpdateChildWindow(m_hWndStatusBar, IDC_PROGRESS);
		}

		_FocusChecker();

		if (_check_flag(MAIN_EX_KILLDIALOG, CMainOption::s_dwMainExtendedStyle))
			CDialogKiller2::KillDialog();
	}

	bool OnDDEOpenFile(const CString& strFileName)
	{
		dmfTRACE(_T("CMainFrame::OnDDEOpenFile(%s)\n"), strFileName);
		OnUserOpenFile(strFileName, DonutGetStdOpenFlag());
		if (CStartUpOption::s_dwActivate) MtlSetForegroundWindow(m_hWnd); // UDT DGSTR ( added by dai
		return true;
	}

// Message map
BEGIN_MSG_MAP(CMainFrame)
	PASS_MSG_MAP_MENUOWNER_TO_EXPMENU(m_FavGroupMenu)
	PASS_MSG_MAP_MENUOWNER_TO_EXPMENU(m_FavoriteMenu)
	CHAIN_MSG_MAP(CUpdateCmdUI<CMainFrame>)

	MESSAGE_HANDLER(WM_CREATE, OnCreate)
	MSG_WM_CLOSE(OnClose)
	MSG_WM_DESTROY(OnDestroy)
	USER_MSG_WM_NEWINSTANCE(OnNewInstance)
	MSG_WM_USER_MDICHILD(OnMDIChild)
	MSG_WM_USER_OPENFILE(OnUserOpenFile)
	MSG_WM_USER_GET_ACTIVE_IWEBBROWSER()

	USER_MSG_WM_UPDATE_TITLEBAR(UpdateTitleBar) // UDT DGSTR
	USER_MSG_WM_UPDATE_EXPBAR(UpdateExpBar) // UDT DGSTR

	MESSAGE_HANDLER(MYWM_NOTIFYICON, OnMyNotifyIcon) // UDT DGSTR ( SystemTrayIcon Handler )
	COMMAND_ID_HANDLER_EX(ID_GET_OUT, OnGetOut) // UDT DGSTR

	COMMAND_ID_HANDLER_EX(ID_DLCTL_COOKIES, OnCookies) // UDT DGSTR
	COMMAND_ID_HANDLER_EX(ID_DLCTL_COOKIES_SESSION, OnCookiesSession) // UDT DGSTR

//	NOTIFY_CODE_HANDLER(TBN_DROPDOWN, OnDropDown) // UDT DGSTR
	
	MESSAGE_HANDLER(MYWM_NOTIFYICON, OnMyNotifyIcon) // UDT DGSTR ( SystemTrayIcon Handler )
	COMMAND_ID_HANDLER_EX(ID_GET_OUT, OnGetOut) // UDT DGSTR

	COMMAND_ID_HANDLER(ID_FILE_OPEN_DEF, OnFileOpenDef) // UDT DGSTR

	MSG_WM_USER_GET_ACTIVE_WINDOW()
	MSG_WM_USER_BROWSER_CAN_SETFOCUS(OnBrowserCanSetFocus)
	MSG_WM_ACTIVATE(OnActivate)
	
	MESSAGE_HANDLER(WM_NOTIFY, OnNotify)// map custom draw notifycation to addressbar

	HANDLE_MESSAGE(WM_MDISETMENU)
	HANDLE_MESSAGE(WM_SETFOCUS)			// default handler will set focus to server window

	MSG_WM_TIMER(OnTimer)
	MSG_WM_MOUSEWHEEL(OnMouseWheel)
//	MSG_WM_PARENTNOTIFY(OnParentNotify)
	DEBUG_CHECK_FOCUS_COMMAND_ID_HANDLER_EX()

	COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
	COMMAND_RANGE_HANDLER(ID_FILE_MRU_FIRST, ID_FILE_MRU_LAST, OnFileRecent)
	COMMAND_ID_HANDLER(ID_FILE_NEW, OnFileNew)
	COMMAND_ID_HANDLER(ID_FILE_OPEN, OnFileOpen)
	COMMAND_ID_HANDLER_EX(ID_FILE_NEW_HOME, OnFileNewHome)
	COMMAND_ID_HANDLER_EX(ID_FILE_NEW_CLIPBOARD, OnFileNewClipBoard)
	COMMAND_ID_HANDLER_EX(ID_FILE_NEW_CLIPBOARD2, OnFileNewClipBoard2)
	COMMAND_ID_HANDLER_EX(ID_FILE_NEW_CLIPBOARD_EX, OnFileNewClipBoardEx)
	COMMAND_ID_HANDLER_EX(ID_FILE_NEW_BLANK, OnFileNewBlank)
	COMMAND_ID_HANDLER_EX(ID_FILE_NEW_COPY, OnFileNewCopy)
	COMMAND_ID_HANDLER_EX(ID_FILE_WORKOFFLINE, OnFileWorkOffline)

	COMMAND_ID_HANDLER_EX(ID_EDIT_CUT, OnEditCut)
	COMMAND_ID_HANDLER_EX(ID_EDIT_COPY, OnEditCopy)
	COMMAND_ID_HANDLER_EX(ID_EDIT_PASTE, OnEditPaste)
	COMMAND_ID_HANDLER_EX(ID_EDIT_SELECT_ALL, OnEditSelectAll)
		
	COMMAND_ID_HANDLER_EX(ID_FAVORITE_GROUP_SAVE, OnFavoriteGroupSave)
	COMMAND_ID_HANDLER_EX(ID_FAVORITE_GROUP_ORGANIZE, OnFavoriteGroupOrganize)

	COMMAND_ID_HANDLER_EX(ID_VIEW_REFRESH_ALL, OnViewRefreshAll)
	COMMAND_ID_HANDLER_EX(ID_VIEW_STOP_ALL, OnViewStopAll)
	COMMAND_ID_HANDLER_EX(ID_VIEW_FULLSCREEN, OnViewFullScreen)

	COMMAND_ID_HANDLER_EX(ID_BACKUPOPTION_CHANGED, OnBackUpOptionChanged)

	COMMAND_ID_HANDLER_EX(ID_TAB_LEFT, OnTabLeft)
	COMMAND_ID_HANDLER_EX(ID_TAB_RIGHT, OnTabRight)

	COMMAND_ID_HANDLER(ID_VIEW_TOOLBAR, OnViewToolBar)
	COMMAND_ID_HANDLER(ID_VIEW_ADDRESSBAR, OnViewAddressBar)
	COMMAND_ID_HANDLER(ID_VIEW_LINKBAR, OnViewLinkBar)
	COMMAND_ID_HANDLER(ID_VIEW_TABBAR, OnViewTabBar)
	COMMAND_ID_HANDLER(ID_VIEW_STATUS_BAR, OnViewStatusBar)
	COMMAND_ID_HANDLER(ID_VIEW_GOBUTTON, OnViewGoButton)
	COMMAND_ID_HANDLER(ID_VIEW_TOOLBAR_CUST, OnViewToolBarCust)
	COMMAND_ID_HANDLER(ID_VIEW_TABBAR_MULTI, OnViewTabBarMulti)
	COMMAND_ID_HANDLER(ID_VIEW_ADDBAR_DROPDOWN, OnViewAddBarDropDown)

	COMMAND_ID_HANDLER(ID_VIEW_FAVEXPBAR, OnViewFavExpBar) // dai
// UDT DGSTR
	COMMAND_ID_HANDLER(ID_VIEW_FAVGEXPBAR, OnFavGroupExpBar)
	COMMAND_ID_HANDLER(ID_VIEW_HISTEXPBAR, OnFavHistoryExpBar)
	
	COMMAND_ID_HANDLER(ID_FAVTREE_BAR_STANDARD, OnFavoriteExpBar)
	COMMAND_ID_HANDLER(ID_FAVTREE_BAR_GROUP, OnFavGroupExpBar)
	COMMAND_ID_HANDLER(ID_FAVTREE_BAR_USER, OnFavUserExpBar)
	COMMAND_ID_HANDLER(ID_FAVTREE_BAR_MYCOMPUTER, OnFavMyCompExpBar)
	COMMAND_ID_HANDLER(ID_FAVTREE_BAR_HISTORY, OnFavHistoryExpBar)
// ENDE
	COMMAND_ID_HANDLER(ID_PANE_CLOSE, OnViewFavExpBar) // dai
	
	COMMAND_ID_HANDLER_EX(ID_VIEW_OPTION_DONUT, OnViewOptionDonut)
	COMMAND_ID_HANDLER_EX(ID_FAVORITE_ORGANIZE, OnFavoriteOrganize)

	COMMAND_ID_HANDLER_EX(ID_SETFOCUS_ADDRESSBAR, OnSetFocusAddressBar)

	COMMAND_ID_HANDLER(ID_WINDOW_CLOSE_ALL, OnWindowCloseAll)

	COMMAND_ID_HANDLER(ID_VIEW_REFRESH_FAVBAR, OnViewRefreshFavBar)
	COMMAND_ID_HANDLER(ID_VIEW_IDLE, OnViewIdle)
	COMMAND_ID_HANDLER_EX(ID_VIEW_HOME, OnViewHome)

	COMMAND_ID_HANDLER_EX(ID_REGISTER_AS_BROWSER, OnRegisterAsBrowser)

	CHAIN_COMMANDS_MEMBER(m_MainOption)
//	CHAIN_COMMANDS_MEMBER(m_secZone)
	CHAIN_COMMANDS(CWindowMenuCommandHandler<CMainFrame>)
	CHAIN_COMMANDS(CAppCommandHandler<CMainFrame>)
	CHAIN_COMMANDS_TO_EXPLORERBAR(m_ExplorerBar)

	CHAIN_MDI_CHILD_COMMANDS()
	COMMAND_ID_HANDLER_EX(ID_VIEW_OPTION, OnViewOption)

	CHAIN_MSG_MAP(CDDEMessageHandler<CMainFrame>)
	CHAIN_MSG_MAP(CMDIFrameTitleUpsideDownMessageHandlerWeb<CMainFrame>)
	CHAIN_MSG_MAP(CHelpMessageLine<CMainFrame>)
	CHAIN_MSG_MAP(CMSMouseWheelMessageHandler<CMainFrame>)
	CHAIN_MSG_MAP(CReBarVerticalBorder<CMainFrame>)

	REFLECT_CHEVRONPUSHED_NOTIFICATION()
	CHAIN_MSG_MAP(baseClass)
	REFLECT_NOTIFICATIONS()// must be last
ALT_MSG_MAP(1)
	MESSAGE_HANDLER(WM_ERASEBKGND, OnMDIClientEraseBkgnd)
	MESSAGE_HANDLER(WM_SIZE, OnMDIClientSize)
END_MSG_MAP()

// Message handlers

// UDT DGSTR
	LRESULT UpdateTitleBar(LPCTSTR lpszStatusBar, DWORD dwReserved)
	{
		if (!::IsWindowVisible(m_hWndStatusBar))
			UpdateTitleBarUpsideDown(lpszStatusBar);
		return 0;
	}
// ENDE

// UDT DGSTR
	LRESULT UpdateExpBar(LPCTSTR lpszExpBar, DWORD dwReserved)
	{
		m_ExplorerBar.SetTitle(lpszExpBar);
		return 0;
	}
// ENDE

// UDT DGSTR
//	LRESULT OnDropDown(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
/*	{
			RECT      rc;
			int iItem;
			LPNMTOOLBAR lpnmTB = (LPNMTOOLBAR)pnmh;
			iItem = lpnmTB->iItem;

			SendMessage(lpnmTB->hdr.hwndFrom, TB_GETRECT,
                     (WPARAM)lpnmTB->iItem, (LPARAM)&rc);
			::MapWindowPoints(lpnmTB->hdr.hwndFrom,
                         HWND_DESKTOP, (LPPOINT)&rc, 2);                        
			CMenu menu;
			CMenuHandle menuHandle;
			if (iItem == ID_VIEW_BACK){
				menu.LoadMenu(IDR_GO_BACK);
				menuHandle = menu.GetSubMenu(0);
			}
			else if (iItem == ID_VIEW_FORWARD){
				menu.LoadMenu(IDR_GO_BACK);
				menuHandle = menu.GetSubMenu(1);
			}
			else if (iItem == ID_VIEW_FAVEXPBAR){
				menu.LoadMenu(IDR_MENU_FAVTREE_BAR);
				menuHandle = menu.GetSubMenu(0);
			}
			MTLVERIFY(menuHandle.TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_VERTICAL,
					rc.left, rc.bottom, m_hWnd));
			DestroyMenu(menuHandle);
			return TBDDRET_DEFAULT;
	}
// ENDE
*/
// UDT DGSTR
	void OnGetOut(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (IsWindowVisible()){
			TrayMessage(m_hWnd, NIM_ADD, TM_TRAY, IDR_MAINFRAME, _T("Donut R")); 
			ShowWindow(SW_HIDE);
			Sleep(100); // UDT TODO
		}
		else{ // just db F9 press , come here :p 
			ShowWindow(SW_NORMAL); 
			TrayMessage(m_hWnd, NIM_DELETE, TM_TRAY, 0, NULL); 
		}
	}
// ENDE
// UDT DGSTR ( hide window & display icon )
	LRESULT OnMyNotifyIcon(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		if(IsWindowVisible())
			return -1;
		else {
			ShowWindow(SW_NORMAL);
			TrayMessage(m_hWnd, NIM_DELETE, TM_TRAY, 0, NULL);
			return 0;
		}
	}
// ENDE

	// alternates OpenFile
	HWND OnUserOpenFile(LPCTSTR lpszFile, DWORD openFlag)
	{
		CString strFile(lpszFile);

		MtlRemoveStringHeaderAndFooter(strFile);
		
		if ( !MtlIsProtocol(strFile, _T("http")) &&
			 !MtlIsProtocol(strFile, _T("https")) )
		{
			if (MtlPreOpenFile(strFile))
				return NULL;// handled
		}
		
		MtlParseIntenetShortcutFile(strFile);

		if (strFile.GetLength() > INTERNET_MAX_PATH_LENGTH)
			return NULL;

		if (strFile == _T("javascript:location.reload()"))
			return NULL;

		// dfg files
		if (MtlIsExt(strFile, _T(".dfg"))) {
			if (!(CMainOption::s_dwMainExtendedStyle & MAIN_EX_NOCLOSEDFG))
				_LoadGroupOption(strFile, true);
			else
				_LoadGroupOption(strFile, false);
			return NULL;
		}


		HWND hWndActive = MDIGetActive();

		int nCmdShow = _check_flag(D_OPENFILE_ACTIVATE, openFlag) ? -1 : SW_SHOWNOACTIVATE;

		if (hWndActive == NULL) {	// no window yet
			nCmdShow = -1;			// always default
		}

		if (hWndActive != NULL && _check_flag(D_OPENFILE_NOCREATE, openFlag)) {
			CWebBrowser2 browser = DonutGetIWebBrowser2(hWndActive);
			if (!browser.IsBrowserNull()) {
				browser.Navigate2(strFile);

				if (!_check_flag(D_OPENFILE_NOSETFOCUS, openFlag)) {
					// reset focus
					::SetFocus(NULL);
					MtlSendCommand(hWndActive, ID_VIEW_SETFOCUS);
				}

				return NULL;
				//return hWndActive;
			}
		}
	
		CChildFrame* pChild = CChildFrame::NewWindow(m_hWndMDIClient, m_MDITab, m_AddressBar);
		if (pChild == NULL)
			return NULL;

		pChild->ActivateFrame(nCmdShow);

		if (!strFile.IsEmpty())
			pChild->Navigate2(strFile);

		if (!_check_flag(D_OPENFILE_NOSETFOCUS, openFlag)) {
			if (MDIGetActive() == pChild->m_hWnd) {// a new window activated, so automatically set focus
				// reset focus
				::SetFocus(NULL);
				MtlSendCommand(pChild->m_hWnd, ID_VIEW_SETFOCUS);
			}
			else {
				// It's reasonable not to touch a current focus.
			}
		}

		return pChild->m_hWnd;
	}

//////////////////////////////////////////////////////////////////
// to avoid the flicker on resizing
	LRESULT OnMDIClientEraseBkgnd(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		HWND hWndActive = MDIGetActive();
		if (CMainOption::s_bTabMode && hWndActive != NULL)
			return 1;// no need to erase it

		bHandled = FALSE;
		return 0;
	}

	LRESULT OnMDIClientSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		BOOL bMaximized = FALSE;
		HWND hWndActive = MDIGetActive(&bMaximized);
		if (CMainOption::s_bTabMode || hWndActive == NULL || bMaximized == FALSE) {// pass to default
			bHandled = FALSE;
			return 1;
		}

		// NOTE. If you do the following, you can avoid the flicker on resizing.
		//       I can't understand why it is effective...
		//
		//       But still you can get a glimpse of other mdi child window's frames.
		//       I guess it's MDI's bug, but I can't get the way MFC fixed.
		CWindow wndActive(hWndActive);
		CWindow wndView = wndActive.GetDlgItem(ID_DONUTVIEW);
		wndActive.ModifyStyle(0, WS_CLIPCHILDREN);// add WS_CLIPCHILDREN
		wndView.ModifyStyle(0, WS_CLIPCHILDREN);
		LRESULT lRet = m_wndMDIClient.DefWindowProc(uMsg, wParam, lParam);
		m_wndMDIClient.UpdateWindow();
		wndActive.ModifyStyle(WS_CLIPCHILDREN, 0);
		wndView.ModifyStyle(WS_CLIPCHILDREN, 0);
		return lRet;
	}
//////////////////////////////////////////////////////////////////
// custom draw of addressbar
	LRESULT OnNotify(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;

		LPNMHDR lpnmh = (LPNMHDR)lParam;
		LRESULT lRet = 0;

		if(lpnmh->code == NM_CUSTOMDRAW && lpnmh->hwndFrom == m_hWndToolBar) {// from rebar
			lRet = m_AddressBar.OnCustomDraw(0, lpnmh, bHandled);
		}

		return lRet;
	}

//////////////////////////////////////////////////////////////////
// the custom message from MDI child
	void OnMDIChild(HWND hWnd, UINT nCode)
	{
		SetMsgHandled(FALSE);
		if (nCode ==  MDICHILD_USER_ALLCLOSED) {
			m_AddressBar.SetWindowText(_T(""));
			::SetWindowText(m_hWndStatusBar, _T("fB"));
			m_CmdBar.EnableButton(_nPosWindowMenu, false);
		}
		else if (nCode == MDICHILD_USER_FIRSTCREATED) {
			m_CmdBar.EnableButton(_nPosWindowMenu, true);
		}
		else if (nCode == MDICHILD_USER_ACTIVATED) {
			OnIdle();		// make sure when cpu is busy
		}
	}

	BOOL OnBrowserCanSetFocus()
	{// asked by browser
		HWND hWndFocus = ::GetFocus();
		if (hWndFocus == NULL)
			return TRUE;

		if (::IsChild(m_hWndToolBar, hWndFocus))
			return FALSE;

		if (m_ExplorerBar.IsChild(hWndFocus))
			return FALSE;

		return TRUE;
	}

	void OnParentNotify(UINT fwEvent, UINT idChild, LPARAM lParam)
	{
		if (fwEvent != WM_RBUTTONDOWN)
			return;

		CPoint pt(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		ClientToScreen(&pt);	
		
		CRect rc;
		::GetClientRect(m_hWndToolBar, &rc);
		CPoint ptRebar = pt; ::ScreenToClient(m_hWndToolBar, &ptRebar);
		if (!rc.PtInRect(ptRebar))			// not on rebar
			return;

		HWND hWnd = ::WindowFromPoint(pt);
		if (hWnd == m_MDITab.m_hWnd) {	// on tab bar
			CPoint ptTab = pt; m_MDITab.ScreenToClient(&ptTab);
			if (m_MDITab.HitTest(ptTab) != -1)
				return;
		}

/*		if (idChild == IDC_MDITAB) {
			CPoint pt2 = pt;
			ClientToScreen(&pt2);
			m_MDITab.ScreenToClient(&pt2);

			if (m_MDITab.HitTest(pt2) != -1)
				return;
		}
*/
		CMenuHandle menuView = ::GetSubMenu(m_CmdBar.GetMenu(), _nPosViewMenu);
		CMenuHandle menu = menuView.GetSubMenu(0);
	
//		CPoint ptCursor; ::GetCursorPos(&ptCursor);
		menu.TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_RIGHTBUTTON, pt.x, pt.y, m_hWnd, NULL);
	}

	void OnDestroy()
	{
		SetMsgHandled(FALSE);

		MtlSendCommand(m_hWnd, ID_VIEW_STOP_ALL);// added by DOGSTORE

		// Note. The script error dialog makes the app hung up.
		//       I can't find the reason, but I can avoid it by closing
		//       windows explicitly.
		MtlCloseAllMDIChildren(m_hWndMDIClient);

		CCriticalIdleTimer::UninstallCriticalIdleTimer();

		// what can I trust?
		ATLASSERT(::IsMenu(m_hMenu));
		::DestroyMenu(m_hMenu);

		RevokeDragDrop();
	}

	void OnActivate(UINT nState, BOOL bMinimized, HWND hWndOther)
	{
		if (nState == WA_INACTIVE) {
			m_hWndFocus = ::GetFocus();
		}
		else {
			if (m_hWndFocus)
				::SetFocus(m_hWndFocus);
		}
	}

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		CFavoritesMenuOption::Add(m_hWnd);
	
		//////////////////////////////////////////////////////////////////
		// initialize menus' infomation
		CMenuHandle menu = m_hMenu;
		CMenuHandle menuFav = menu.GetSubMenu(_nPosFavoriteMenu);
		m_FavoriteMenu.InstallExplorerMenu(menuFav);

		CMenuHandle menuGroup = menuFav.GetSubMenu(_nPosFavGroupMenu);
		m_FavGroupMenu.InstallExplorerMenu(menuGroup);

		// MRU list
		CMainOption::SetMRUMenuHandle(menu, _nPosMRU);

		//////////////////////////////////////////////////////////////////
		// create command bar window
		SetMenu(NULL);// remove menu
		HWND hWndCmdBar = m_CmdBar.Create(m_hWnd, rcDefault, NULL, MTL_SIMPLE_CMDBAR2_PANE_STYLE, 0, ATL_IDW_COMMAND_BAR);
		m_CmdBar.AttachMenu(m_hMenu);
		m_CmdBar.EnableButton(_nPosWindowMenu, false);

		//////////////////////////////////////////////////////////////////
		// create toolbar
		HWND hWndToolBar = m_ToolBar.DonutToolBar_Create(m_hWnd);
		ATLASSERT(::IsWindow(hWndToolBar));
		m_ToolBar.DonutToolBar_SetFavoritesMenu(menuFav);

		//////////////////////////////////////////////////////////////////
		// create addressbar
		HWND hWndAddressBar = m_AddressBar.Create(m_hWnd, IDC_ADDRESSBAR, ID_VIEW_GO,
			IDB_GOBUTTON, IDB_GOBUTTON_HOT, 16, 16, RGB(255, 0, 255));
		ATLASSERT(::IsWindow(hWndAddressBar));
		
		//////////////////////////////////////////////////////////////////
		// create tabctrl
		HWND hWndMDITab = m_MDITab.Create(m_hWnd, CRect(0, 0, 200, 20), _T("Donut MDITab"),
			WS_CHILD | WS_VISIBLE, NULL, IDC_MDITAB);
		m_MDITab.LoadMenu(IDR_MENU_TAB);
		m_MDITab.LoadConnectingAndDownloadingImageList(IDB_MDITAB, 6, 6, RGB(255, 255, 255));

		//////////////////////////////////////////////////////////////////
		// create link bar
		HWND hWndLinkBar = m_LinkBar.Create(m_hWnd, rcDefault, NULL, ATL_SIMPLE_TOOLBAR_PANE_STYLE, 0, IDC_LINKBAR);

		//////////////////////////////////////////////////////////////////
		// create rebar
		CreateSimpleReBar(MTL_SIMPLE_REBAR_STYLE | RBS_DBLCLKTOGGLE);
//		CWindow(m_hWndToolBar).ModifyStyleEx(0, WS_EX_CLIENTEDGE);

		//////////////////////////////////////////////////////////////////
		// create statusbar //CreateSimpleStatusBar();
		m_wndStatusBar.Create(m_hWnd);
		m_hWndStatusBar = m_wndStatusBar.m_hWnd;
		int nPanes[] = { ID_DEFAULT_PANE, ID_PROGRESS_PANE };
		m_wndStatusBar.SetPanes(nPanes, 2, false);

		//////////////////////////////////////////////////////////////////
		// create prgress bar
		m_wndProgress.Create(m_hWndStatusBar, NULL, NULL, WS_CHILD | WS_CLIPSIBLINGS    
			| PBS_SMOOTH, 0, IDC_PROGRESS);
		m_wndProgress.ModifyStyleEx(WS_EX_STATICEDGE, 0);

		//////////////////////////////////////////////////////////////////
		CIniSection pr;

		//////////////////////////////////////////////////////////////////
		// load band position
		CReBarBandInfo rbis[] = {
			{ 0, hWndCmdBar, ATL_IDW_COMMAND_BAR, 0, NULL, 0 },
			{ 1, hWndToolBar, ATL_IDW_TOOLBAR, RBBS_BREAK, NULL, 0 },
			{ 2, hWndAddressBar, IDC_ADDRESSBAR, RBBS_BREAK, NULL/*_T("AhX(D)")*/, 0 },
			{ 3, hWndMDITab, IDC_MDITAB, RBBS_BREAK, NULL, 0 },
			{ 4, hWndLinkBar, IDC_LINKBAR, RBBS_BREAK, NULL/*_T("N")*/, 0 }, // UDT DGSTR
		};
		pr.Open(_szIniFileName, _T("ReBar"));
		MtlGetProfileReBarBandsState(rbis, rbis + _countof(rbis), pr, *this);
		pr.Close();

		m_CmdBar.RefreshBandIdealSize(m_hWndToolBar);
		m_AddressBar.InitReBarBandInfo(m_hWndToolBar);// if you dislike a header, remove this.

		//////////////////////////////////////////////////////////////////
		// load status bar state
		pr.Open(_szIniFileName, _T("Main"));
		BOOL bStatusBarVisible = TRUE;
		MtlGetProfileStatusBarState(pr, m_hWndStatusBar, bStatusBarVisible);
		m_bStatusBarVisibleUnlessFullScreen = bStatusBarVisible;
		pr.Close();

		//////////////////////////////////////////////////////////////////
		// create mdi client window
		m_menuWindow = ::GetSubMenu(m_hMenu, _nPosWindowMenu);
		CreateMDIClient(m_menuWindow.m_hMenu);
		m_wndMDIClient.SubclassWindow(m_hWndMDIClient);

		// NOTE: If WS_CLIPCHILDREN not set, MDI Client will try erase background over MDI child window,
		//       but as OnEraseBkgnd does nothing, it goes well.
		if (CMainOption::s_bTabMode)
			m_wndMDIClient.ModifyStyle(WS_CLIPCHILDREN, 0);// to avoid flicker, but scary

		//////////////////////////////////////////////////////////////////
		// splitter window
		m_wndSplit.SetSplitterExtendedStyle(0);
		m_hWndClient = m_wndSplit.Create(m_hWnd, rcDefault, NULL,
			WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN);

		//////////////////////////////////////////////////////////////////
		// pane container
		m_ExplorerBar.Create(m_hWndClient);
		m_ExplorerBar.DonutExplorerBar_Init(m_hWndMDIClient);

		//////////////////////////////////////////////////////////////////
		// MDIClient misc
		m_mcCmdBar.InstallAsMDICmdBar(hWndCmdBar, m_hWndMDIClient, CMainOption::s_bTabMode);
		m_mcToolBar.InstallAsStandard(hWndToolBar, m_hWnd, true, ID_VIEW_FULLSCREEN);

		m_MDITab.SetMDIClient(m_hWndMDIClient);

		//////////////////////////////////////////////////////////////////
		// set up UI
		CmdUIAddToolBar(hWndToolBar);

		//m_ToolBar.SetExtendedStyle(TBSTYLE_EX_DRAWDDARROWS); // UDT DGSTR  ( set Extend Style , EX_DRAWDDARROWS ) // comment out by dai
		
		// UDT DGSTR ( set User Agent )
		::UrlMkSetSessionOption(URLMON_OPTION_USERAGENT , 
			(void *)CDLControlOption::s_szUserAgent , lstrlen(CDLControlOption::s_szUserAgent) , 0);
		// ENDE

		// message loop	
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		pLoop->AddMessageFilter(this);
		pLoop->AddIdleHandler(this);

		// read general settings
		_ReadProfile();
		// for dragging files
		RegisterDragDrop();// DragAcceptFiles();

		_ShowBandsFullScreen(true, true);

		CCriticalIdleTimer::InstallCriticalIdleTimer(m_hWnd, ID_VIEW_IDLE);

		return 0;
	}

	LRESULT OnMouseWheel(UINT fwKeys, short zDelta, CPoint point)
	{
		// I don't have a wheel mouse...
		if ( (m_MDITab.GetMDITabExtendedStyle() & MTB_EX_WHEEL) &&
			  MtlIsBandVisible(m_hWndToolBar, IDC_MDITAB) ) {
			CRect rcTab;
			m_MDITab.GetWindowRect(&rcTab);
			if (rcTab.PtInRect(point)) {
				if (zDelta > 0) {
					m_MDITab.LeftTab();
				}
				else {
					m_MDITab.RightTab();
				}
				SetMsgHandled(TRUE);
				return 0;
			}
		}

		SetMsgHandled(FALSE);
		return 1;
	}

	LRESULT OnFileRecent(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// get file name from the MRU list
		TCHAR szFile[INTERNET_MAX_PATH_LENGTH];
		if(CMainOption::s_pMru->GetFromList(wID, szFile))
		{
			if(CMainOption::s_pMru->RemoveFromList(wID)) // UDT DGSTR
				OnUserOpenFile(szFile, DonutGetStdOpenCreateFlag()/*DonutGetStdOpenFlag()*/); // Force New Window // UDT DGSTR
		}
		else
		{
			::MessageBeep(MB_ICONERROR);
		}

		return 0;
	}

	LRESULT OnViewRefreshFavBar(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_FavoriteMenu.RefreshMenu();
		m_FavGroupMenu.RefreshMenu();

		return 0;
	}

	LRESULT OnViewIdle(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		_OnIdleCritical();

		return 0;
	}

	void OnClose()
	{
		SetMsgHandled(FALSE);

		if (!CDonutConfirmOption::OnDonutExit(m_hWnd)) {
			SetMsgHandled(TRUE);
			return;
		}

		if (_IsFullScreen()) {
			CMainOption::s_dwMainExtendedStyle |= MAIN_EX_FULLSCREEN;// save style
			_ShowBandsFullScreen(FALSE);// restore bands position
		}
		else {
			CMainOption::s_dwMainExtendedStyle &= ~MAIN_EX_FULLSCREEN;
		}
		
		if (m_nBackUpTimerID != 0)
			KillTimer(m_nBackUpTimerID);

		m_mcCmdBar.Uninstall();
		m_mcToolBar.Uninstall();

		_WriteProfile();

		CMainOption::s_bAppClosing = true;
	}

	void OnNewInstance(ATOM nAtom)
	{
		TCHAR szBuff[4096];
		if (::GlobalGetAtomName(nAtom, szBuff, 4096) != 0) {
			dmfTRACE(_T("CMainFrame::OnNewInstance: %s\n"), szBuff);
			// UDT DGSTR ( Shell Open on dup running )
			// atom table can only 255 charactor ... so far. :p
			// fixed by JOBBY . thx ( local check , remove quota.
			//OnUserOpenFile(szBuff, DonutGetStdOpenFlag());
			CString strURLCol(szBuff);
			if (strURLCol.Left(4).CompareNoCase(_T("http"))==0)
			{
				int nPos = 0;
				while (1)
				{
					nPos = strURLCol.Find(_T(" "));
					if ( nPos == -1 )
					{
						OnUserOpenFile(strURLCol , DonutGetStdOpenCreateFlag());
						break;
					}
					else
						OnUserOpenFile(strURLCol.Left(nPos) , DonutGetStdOpenCreateFlag());
				strURLCol = strURLCol.Mid(nPos+1);
				}
			}
			else
			{
				// remove first & last double-quotation
				if(strURLCol.GetAt(0)=='"')
					strURLCol=strURLCol.Mid(1, strURLCol.GetLength()-2);
				OnUserOpenFile(strURLCol, DonutGetStdOpenFlag());
			}
			// ENDE
			::GlobalDeleteAtom(nAtom);
			if (!(CMainOption::s_dwMainExtendedStyle & MAIN_EX_NOACTIVATE))
				MtlSetForegroundWindow(m_hWnd);
		}
	}

	LRESULT OnFileNew(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		if (CFileNewOption::s_dwFlags == FILENEW_BLANK)
			OnFileNewBlank(0, 0, 0);
		else if (CFileNewOption::s_dwFlags == FILENEW_COPY) {
			if (MDIGetActive() != NULL)
				SendMessage(WM_COMMAND, MAKEWPARAM(ID_FILE_NEW_COPY, 0), 0);
			else
				OnFileNewBlank(0, 0, 0);
		}
		else if (CFileNewOption::s_dwFlags == FILENEW_HOME)
			OnFileNewHome(0, 0, 0);
		else
			ATLASSERT(FALSE);

		return 0;
	}

	void OnFileNewHome(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		HWND hWndChild = OnUserOpenFile(NULL, DonutGetStdOpenActivateFlag());
		if (hWndChild) {
			CWebBrowser2 browser = DonutGetIWebBrowser2(hWndChild);
			if (browser.m_spBrowser != NULL)
				browser.GoHome();
		}
	}

	void OnViewHome(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		bool bNew = ::GetAsyncKeyState(VK_CONTROL) < 0 || ::GetAsyncKeyState(VK_SHIFT) < 0;
		if (bNew || MDIGetActive() == NULL) {
			OnFileNewHome(0, 0, 0);
		}
		else {
			SetMsgHandled(FALSE);
		}
	}

	void OnRegisterAsBrowser(WORD wNotifyCode, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (wNotifyCode == NM_ON) {
			MtlForEachMDIChild(m_hWndMDIClient, CSendCommand(ID_REGISTER_AS_BROWSER, NM_ON));
		}
		else if (wNotifyCode == NM_OFF) {
			MtlForEachMDIChild(m_hWndMDIClient, CSendCommand(ID_REGISTER_AS_BROWSER, NM_OFF));
		}
		else if (!_check_flag(MAIN_EX_REGISTER_AS_BROWSER, CMainOption::s_dwMainExtendedStyle)) {
			CMainOption::s_dwMainExtendedStyle |= MAIN_EX_REGISTER_AS_BROWSER;
			MtlForEachMDIChild(m_hWndMDIClient, CSendCommand(ID_REGISTER_AS_BROWSER, NM_ON));
		}
		else {
			CMainOption::s_dwMainExtendedStyle &= ~MAIN_EX_REGISTER_AS_BROWSER;
			MtlForEachMDIChild(m_hWndMDIClient, CSendCommand(ID_REGISTER_AS_BROWSER, NM_OFF));
		}
	}

	void OnFileNewClipBoard(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		CString strText = MtlGetClipboardText();
		if (strText.IsEmpty())
			return;

		OnUserOpenFile(strText, DonutGetStdOpenActivateFlag());
	}

	void OnFileNewClipBoard2(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		CString strText = MtlGetClipboardText();
		if (strText.IsEmpty())
			return;

		OnUserOpenFile(strText, DonutGetStdOpenFlag());// allow the option
	}

	void OnFileNewClipBoardEx(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		m_ExplorerBar.m_ClpBar.OpenClipboardUrl();
	}

	void OnFileNewCopy(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		HWND hWndActive = MDIGetActive();
		if (hWndActive) {
			CWebBrowser2 browser = DonutGetIWebBrowser2(hWndActive);
			if (browser.m_spBrowser != NULL) {
				CString strURL = browser.GetLocationURL();
				if (!strURL.IsEmpty())
					OnUserOpenFile(strURL, DonutGetStdOpenActivateFlag());
			}
		}
	}

	void OnFileNewBlank(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		OnUserOpenFile(_T("about:blank"), DonutGetStdOpenActivateFlag());
	}

	void OnFileWorkOffline(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		MtlSetGlobalOffline(!MtlIsGlobalOffline());
		UpdateTitleBarUpsideDown();
	}

	LRESULT OnFileOpen(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		dmfTRACE(_T("CMainFrame::OnFileOpen\n"));
		COpenURLDlg dlg;
		if (dlg.DoModal() == IDOK && !dlg.m_strEdit.IsEmpty()) {
			OnUserOpenFile(dlg.m_strEdit, DonutGetStdOpenFlag());
		}
		return 0;
	}
	// UDT DGSTR
	LRESULT OnFileOpenDef(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{	
		CString strPath = MtlGetModuleFileName();

		strPath = strPath.Left(strPath.ReverseFind('\\')) + _T("\\Default.dfg");

		ATLTRACE2(atlTraceUser, 4, _T("CMainFrame::OnFileOpenDef\n"));
		OnUserOpenFile(strPath, DonutGetStdOpenCreateFlag());
		return 0;
	}
	// ENDE
	LRESULT OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		PostMessage(WM_CLOSE);
		return 0;
	}

	void OnEditCut(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{	
		CEdit edit = m_AddressBar.GetEditCtrl();
		if (::GetFocus() != edit) {
			SetMsgHandled(FALSE);
			return;
		}

		edit.Cut();
	}

	void OnEditCopy(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		CEdit edit = m_AddressBar.GetEditCtrl();
		if (::GetFocus() != edit) {
			SetMsgHandled(FALSE);
			return;
		}

		edit.Copy();
	}

	void OnEditPaste(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		CEdit edit = m_AddressBar.GetEditCtrl();
		if (::GetFocus() != edit) {
			SetMsgHandled(FALSE);
			return;
		}

		edit.Paste();
	}

	void OnEditSelectAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		CEdit edit = m_AddressBar.GetEditCtrl();
		if (::GetFocus() != edit) {
			SetMsgHandled(FALSE);
			return;
		}

		edit.SetSel(0, -1);
	}

	LRESULT OnViewToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// It sucks, band index is dynamic.
		// ::SendMessage(m_hWndToolBar, RB_SHOWBAND, 0, bNew);	// toolbar is band #0
		MtlToggleBandVisible(m_hWndToolBar, ATL_IDW_TOOLBAR);
		UpdateLayout();
		return 0;
	}
	LRESULT OnViewAddressBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		MtlToggleBandVisible(m_hWndToolBar, IDC_ADDRESSBAR);
		UpdateLayout();
		return 0;
	}
	LRESULT OnViewLinkBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		MtlToggleBandVisible(m_hWndToolBar, IDC_LINKBAR);
		UpdateLayout();
		return 0;
	}

	LRESULT OnViewGoButton(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		if (_check_flag(ABR_EX_GOBTNVISIBLE, m_AddressBar.GetAddressBarExtendedStyle()))
			m_AddressBar.ModifyAddressBarExtendedStyle(ABR_EX_GOBTNVISIBLE, 0);
		else
			m_AddressBar.ModifyAddressBarExtendedStyle(0, ABR_EX_GOBTNVISIBLE);

		return 0;
	}
	LRESULT OnViewTabBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		MtlToggleBandVisible(m_hWndToolBar, IDC_MDITAB);
		UpdateLayout();
		return 0;
	}
	
	LRESULT OnViewTabBarMulti(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		DWORD dwOldStyle = m_MDITab.GetMDITabExtendedStyle();
		if (dwOldStyle & MTB_EX_MULTILINE)
			dwOldStyle &= ~MTB_EX_MULTILINE;
		else
			dwOldStyle |= MTB_EX_MULTILINE;

		m_MDITab.SetMDITabExtendedStyle(dwOldStyle);
		return 0;
	}

	LRESULT OnViewAddBarDropDown(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		if (m_AddressBar.GetDroppedStateEx())
			m_AddressBar.ShowDropDown(FALSE);
		else
			m_AddressBar.ShowDropDown(TRUE);

		return 0;
	}

	LRESULT OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		BOOL bNew = !::IsWindowVisible(m_hWndStatusBar);
		m_bStatusBarVisibleUnlessFullScreen = bNew;// save
		::ShowWindow(m_hWndStatusBar, bNew ? SW_SHOWNOACTIVATE : SW_HIDE);
		UpdateLayout();
		return 0;
	}
	LRESULT OnViewToolBarCust(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_ToolBar.StdToolBar_Customize();
		return 0;
	}

	LRESULT OnViewFavExpBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		if (m_wndSplit.GetSinglePaneMode() == SPLIT_PANE_RIGHT) {	//left hidden
			m_ExplorerBar.ShowBar(true);
			m_wndSplit.SetSinglePaneMode();
			m_ExplorerBar.SetTitle(_T("ftHg")); // UDT DGSTR
			// m_wndSplit.SetSplitterPanes(m_FavTree, m_hWndMDIClient);
		}
		else {
			m_ExplorerBar.ShowBar(false);
			m_wndSplit.SetSinglePaneMode(SPLIT_PANE_RIGHT);
		}

		return 0;
	}
	// UDT DGSTR
	LRESULT OnFavoriteExpBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_ExplorerBar.ShowBar(true);
		MtlSendCommand(m_ExplorerBar.m_FavBar.m_hWnd, ID_FAVTREE_BAR_STANDARD);
		m_wndSplit.SetSinglePaneMode();
		m_ExplorerBar.SetTitle(_T("Cɓ"));

		return 0;
	}
	LRESULT OnFavHistoryExpBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_ExplorerBar.ShowBar(true);
		MtlSendCommand(m_ExplorerBar.m_FavBar.m_hWnd, ID_FAVTREE_BAR_HISTORY);
		m_wndSplit.SetSinglePaneMode();
		m_ExplorerBar.SetTitle(_T(""));

		return 0;
	}
	LRESULT OnFavGroupExpBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_ExplorerBar.ShowBar(true);
		MtlSendCommand(m_ExplorerBar.m_FavBar.m_hWnd, ID_FAVTREE_BAR_GROUP);
		m_wndSplit.SetSinglePaneMode();
		m_ExplorerBar.SetTitle(_T("CɓO[v"));

		return 0;
	}
	LRESULT OnFavUserExpBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_ExplorerBar.ShowBar(true);
		MtlSendCommand(m_ExplorerBar.m_FavBar.m_hWnd, ID_FAVTREE_BAR_USER);
		m_wndSplit.SetSinglePaneMode();
		m_ExplorerBar.SetTitle(_T("[U`"));

		return 0;
	}
	LRESULT OnFavMyCompExpBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_ExplorerBar.ShowBar(true);
		MtlSendCommand(m_ExplorerBar.m_FavBar.m_hWnd, ID_FAVTREE_BAR_MYCOMPUTER);
		m_wndSplit.SetSinglePaneMode();
		m_ExplorerBar.SetTitle(_T("fXNgbv"));

		return 0;
	}
	// ENDE
	void OnFavoriteOrganize(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		bool bOldShell = _check_flag(MAIN_EX_ORGFAVORITEOLDSHELL, CMainOption::s_dwMainExtendedStyle);
//		CString strPath = DonutGetFavoritesFolder();
//		MtlOrganizeFavorite(m_hWnd, bOldShell, strPath);
		MtlOrganizeFavorite(m_hWnd, bOldShell, DonutGetFavoritesFolder());
	}

	void OnBackUpOptionChanged(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (m_nBackUpTimerID != 0) {
			KillTimer(m_nBackUpTimerID);
			m_nBackUpTimerID = 0;
		}

		if (CMainOption::s_dwMainExtendedStyle & MAIN_EX_BACKUP)
			m_nBackUpTimerID = SetTimer(1, 1000*60*CMainOption::s_dwBackUpTime);
	}

	void OnTimer(UINT nIDEvent, TIMERPROC*)
	{
		if (m_nBackUpTimerID != nIDEvent) {
			SetMsgHandled(FALSE);
			return;
		}

		_SaveGroupOption(CStartUpOption::GetDefaultDFGFilePath());
	}

	void OnViewOptionDonut(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		CMainPropertyPage pageMain(m_hWnd);
		CDLControlPropertyPage pageDLC;
		CMDITabPropertyPage pageTab(&m_MDITab);
		CDonutAddressBarPropertyPage pageAddress(m_AddressBar);
		CDonutFavoritesMenuPropertyPage pageFav;
		CFileNewPropertyPage pageFileNew;
		CStartUpPropertyPage pageStartUp;

		CString strURL, strTitle;
		HWND hWndActive = MDIGetActive();
		if (hWndActive) {
			CWebBrowser2 browser = DonutGetIWebBrowser2(hWndActive);
			if (browser.m_spBrowser != NULL) {
				strURL = browser.GetLocationURL();
				strTitle = MtlGetWindowText(hWndActive);// pChild->get_LocationName();
			}
		}

		CIgnoredURLsPropertyPage pageURLs(strURL);
		CCloseTitlesPropertyPage pageTitles(strTitle);
		CDonutExecutablePropertyPage pageExe;
		CDonutConfirmPropertyPage pageCnf;

		CCenterPropertySheet sheet(_T("Donut̃IvV"));
		sheet.AddPage(pageMain);
		sheet.AddPage(pageTab);
		sheet.AddPage(pageAddress);
		sheet.AddPage(pageFileNew);
		sheet.AddPage(pageStartUp);
		sheet.AddPage(pageDLC);
		sheet.AddPage(pageURLs);
		sheet.AddPage(pageTitles);
		sheet.AddPage(pageFav);
		sheet.AddPage(pageExe);
		sheet.AddPage(pageCnf);

		sheet.DoModal();
//			m_LinkBar.m_ExpMenu.SetStyle(m_FavoriteMenu.GetStyle());
//		}
	}

	void OnViewOption(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (MDIGetActive() == NULL)
			MtlShowInternetOptions();
	}

	LRESULT OnWindowCloseAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		if (!CDonutConfirmOption::OnCloseAll(m_hWnd))
			return 0;

		CWaitCursor cur;
		CLockRedrawMDIClient lock(m_hWndMDIClient);
		CLockRedraw lock2(m_MDITab);
		MtlCloseAllMDIChildren(m_hWndMDIClient);
		return 0;
	}

	bool _IsFullScreen()
	{
		if (GetStyle() & WS_CAPTION)
			return false;
		else
			return true;
	}

	struct _Function_FullScreen
	{
		CMainFrame& _wndMain;
		_Function_FullScreen(CMainFrame& wndMain) : _wndMain(wndMain) { }
		bool operator()()
		{
			if (CMainOption::s_dwMainExtendedStyle & MAIN_EX_FULLSCREEN) {
				_wndMain._FullScreen(TRUE);
				return true;
			}

			return false;
		}
	};

	void _ShowBandsFullScreen(BOOL bOn, bool bInit = false)
	{
		static BOOL bOldVisibleToolBar, bOldVisibleAddressBar,
			bOldVisibleGoButton, bOldVisibleMDITab, bOldVisibleLinkBar;

		if (bInit) {
 			bOldVisibleToolBar = MtlIsBandVisible(m_hWndToolBar, ATL_IDW_TOOLBAR);
			bOldVisibleAddressBar = MtlIsBandVisible(m_hWndToolBar, IDC_ADDRESSBAR);
//			bOldVisibleMDITab = MtlIsBandVisible(m_hWndToolBar, IDC_MDITAB);
			bOldVisibleLinkBar = MtlIsBandVisible(m_hWndToolBar, IDC_LINKBAR);
			return;
		}

		if (bOn) {// remove caption
			// save prev visible
 			bOldVisibleToolBar = MtlIsBandVisible(m_hWndToolBar, ATL_IDW_TOOLBAR);
			bOldVisibleAddressBar = MtlIsBandVisible(m_hWndToolBar, IDC_ADDRESSBAR);
//			bOldVisibleMDITab = MtlIsBandVisible(m_hWndToolBar, IDC_MDITAB);
			bOldVisibleLinkBar = MtlIsBandVisible(m_hWndToolBar, IDC_LINKBAR);

			MtlShowBand(m_hWndToolBar, ATL_IDW_COMMAND_BAR, FALSE);// hide it.
			MtlShowBand(m_hWndToolBar, ATL_IDW_TOOLBAR, TRUE);// show it.
			MtlShowBand(m_hWndToolBar, IDC_ADDRESSBAR, FALSE);
//			MtlShowBand(m_hWndToolBar, IDC_MDITAB, TRUE);// show it
			MtlShowBand(m_hWndToolBar, IDC_LINKBAR, FALSE);
			::ShowWindow(m_hWndStatusBar, SW_HIDE);// hide it
		}
		else {
			// restore prev visible
			MtlShowBand(m_hWndToolBar, ATL_IDW_COMMAND_BAR, TRUE);// show it.
			MtlShowBand(m_hWndToolBar, ATL_IDW_TOOLBAR, bOldVisibleToolBar);// show it.
			MtlShowBand(m_hWndToolBar, IDC_ADDRESSBAR, bOldVisibleAddressBar);
//			MtlShowBand(m_hWndToolBar, IDC_MDITAB, bOldVisibleMDITab);// show it
			MtlShowBand(m_hWndToolBar, IDC_LINKBAR, bOldVisibleLinkBar);
			::ShowWindow(m_hWndStatusBar, m_bStatusBarVisibleUnlessFullScreen ? SW_SHOWNOACTIVATE : SW_HIDE);
		}
	}

	void _FullScreen(BOOL bOn)
	{
		static BOOL bOldMaximized;

		_ShowBandsFullScreen(bOn);

		if (bOn) {// remove caption
			// save prev visible
			CWindowPlacement wndpl;
			GetWindowPlacement(&wndpl);
			bOldMaximized = (wndpl.showCmd == SW_SHOWMAXIMIZED);

			ModifyStyle(WS_CAPTION, 0);

			m_mcToolBar.m_bVisible = true;
			if (!bOldMaximized)
				ShowWindow(SW_MAXIMIZE);
			else {
				SetWindowPos(NULL, 0, 0, 0, 0,
					SWP_NOSIZE|SWP_NOMOVE|SWP_NOACTIVATE|SWP_NOZORDER|SWP_FRAMECHANGED);
			}
		}
		else {
			m_mcToolBar.m_bVisible = false;
			// restore prev visible
			ModifyStyle(0, WS_CAPTION);

			if (!bOldMaximized)
				ShowWindow(SW_RESTORE);
			else {
				SetWindowPos(NULL, 0, 0, 0, 0,
					SWP_NOSIZE|SWP_NOMOVE|SWP_NOACTIVATE|SWP_NOZORDER|SWP_FRAMECHANGED);
			}
		}
	}

	void OnViewFullScreen(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (GetStyle() & WS_CAPTION) {// remove caption
			_FullScreen(TRUE);
		}
		else {
			_FullScreen(FALSE);
		}

//		SetWindowPos(NULL, 0, 0, 0, 0,
//			SWP_NOSIZE|SWP_NOMOVE|SWP_NOACTIVATE|SWP_NOZORDER|SWP_FRAMECHANGED);
	}

	void OnFavoriteGroupSave(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (MDIGetActive() == NULL)
			return;

		TCHAR szOldPath[MAX_PATH];// save current directory
		::GetCurrentDirectory(MAX_PATH, szOldPath);

		const TCHAR szFilter[] = _T("Donut Favorite Groupt@C(*.dfg)\0*.dfg\0\0");

		::SetCurrentDirectory(DonutGetFavoriteGroupFolder());

		CFileDialog fileDlg(FALSE, _T("dfg"), NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter);
		fileDlg.m_ofn.lpstrInitialDir = DonutGetFavoriteGroupFolder();
		fileDlg.m_ofn.lpstrTitle = _T("CɓO[v̕ۑ");
		if (fileDlg.DoModal() == IDOK) {
			_SaveGroupOption(fileDlg.m_szFileName);
		}

		// restore current directory
		::SetCurrentDirectory(szOldPath);
	}

	void OnFavoriteGroupOrganize(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		::ShellExecute(m_hWnd, NULL, DonutGetFavoriteGroupFolder(), NULL, NULL, SW_SHOWNORMAL);
	}

	void OnViewRefreshAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		MtlForEachMDIChild(m_hWndMDIClient, CPostCommand(ID_VIEW_REFRESH));
	}

	void OnViewStopAll(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		MtlForEachMDIChild(m_hWndMDIClient, CPostCommand(ID_VIEW_STOP));
	}

	void OnSetFocusAddressBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND hWndCtl)
	{
		::SetFocus(m_AddressBar.GetEditCtrl());
	}

	void OnTabLeft(WORD /*wNotifyCode*/, WORD /*wID*/, HWND hWndCtl)
	{
		m_MDITab.LeftTab();
	}

	void OnTabRight(WORD /*wNotifyCode*/, WORD /*wID*/, HWND hWndCtl)
	{
		m_MDITab.RightTab();
	}

	// UDT DGSTR ( cookies )
	void OnCookies(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (GetSecure(URLACTION_COOKIES))
			SetSecure(URLACTION_COOKIES,URLPOLICY_ALLOW);
		else 
			SetSecure(URLACTION_COOKIES,URLPOLICY_DISALLOW);
	}
	void OnCookiesSession(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (GetSecure(URLACTION_COOKIES_SESSION))
			SetSecure(URLACTION_COOKIES_SESSION,URLPOLICY_ALLOW);
		else
			SetSecure(URLACTION_COOKIES_SESSION,URLPOLICY_DISALLOW);
	}
	// ENDE

	void __AddToAddressBoxUnique(const CString& strURL)
	{
		COMBOBOXEXITEM item;
		item.mask = CBEIF_TEXT;
		item.iItem = 0;
		item.pszText = (LPTSTR)(LPCTSTR)strURL;

		// search the same string
		int nCount = m_AddressBar.GetCount();
		for (int n = 0; n < nCount; ++n) {
			CString str;
			MtlGetLBTextFixed(m_AddressBar, n, str);
			if (strURL == str) {
				m_AddressBar.DeleteItem(n);
				break;
			}
		}

		m_AddressBar.InsertItem(&item);
	}
	
	void __MoveToTopAddressBox(const CString& strURL)
	{
		COMBOBOXEXITEM item;
		item.mask = CBEIF_TEXT;
		item.iItem = 0;
		item.pszText = (LPTSTR)(LPCTSTR)strURL;
		
		// search the same string
		int nCount = m_AddressBar.GetCount();
		for (int n = 0; n < nCount; ++n) {
			CString str;
			MtlGetLBTextFixed(m_AddressBar, n, str);
			if (strURL == str) {
				m_AddressBar.DeleteItem(n);
				m_AddressBar.InsertItem(&item);
				break;
			}
		}
	}

// Implementation
private:
	void _ReadProfile()
	{
		CIniSection pr;

		// read tab
		pr.Open(_szIniFileName, _T("MDITab"));
		MtlGetProfileMDITab(pr, m_MDITab);
		pr.Close();

		OnBackUpOptionChanged(0, 0, 0);
	}

	void _WriteProfile()
	{
		CIniSection pr;

		// save frame and status bar
		pr.Open(_szIniFileName, _T("Main"));
		MtlWriteProfileMainFrameState(pr, m_hWnd);
		MtlWriteProfileStatusBarState(pr, m_hWndStatusBar);
		pr.Close();

		// save MDI tab
		pr.Open(_szIniFileName, _T("MDITab"));
		MtlWriteProfileMDITab(pr, m_MDITab);
		pr.Close();

		// save rebar
		pr.Open(_szIniFileName, _T("ReBar"));
		MtlWriteProfileReBarBandsState(pr, m_hWndToolBar);
		pr.Close();

		// save group options
		_SaveGroupOption(CStartUpOption::GetDefaultDFGFilePath());
	}

// Overrides WTL sucks
public:
	static BOOL AddSimpleReBarBandCtrl(HWND hWndReBar, HWND hWndBand, int nID = 0, LPTSTR lpstrTitle = NULL, BOOL bNewRow = FALSE, int cxWidth = 0, BOOL bFullWidthAlways = FALSE)
	{
		dmfTRACE(_T("CMainFrame::AddSimpleReBarBandCtrl\n"));
		ATLASSERT(::IsWindow(hWndReBar));	// must be already created
		ATLASSERT(::IsWindow(hWndBand));	// must be already created
		MTLASSERT_KINDOF(REBARCLASSNAME, hWndReBar);

		// Set band info structure
		REBARBANDINFO rbBand;
		rbBand.cbSize = sizeof(REBARBANDINFO);
#if (_WIN32_IE >= 0x0400)
		rbBand.fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE | RBBIM_IDEALSIZE;
#else
		rbBand.fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE;
#endif //!(_WIN32_IE >= 0x0400)
		if(lpstrTitle != NULL)
			rbBand.fMask |= RBBIM_TEXT;
#if (_WIN32_IE >= 0x0500)
		if (nID != IDC_ADDRESSBAR && nID != IDC_MDITAB)
			rbBand.fStyle = /*RBBS_CHILDEDGE | shit*/RBBS_USECHEVRON;
		else
			rbBand.fStyle = 0;//RBBS_CHILDEDGE;
		// UDT DGSTR ( too small  ;( )
		if (nID == IDC_MDITAB)
			rbBand.fStyle |= RBBS_CHILDEDGE;
		// ENDE
#else
		rbBand.fStyle = 0;//RBBS_CHILDEDGE;
#endif //!(_WIN32_IE >= 0x0500)
		if(bNewRow)
			rbBand.fStyle |= RBBS_BREAK;

		rbBand.lpText = lpstrTitle;
		rbBand.hwndChild = hWndBand;
		rbBand.wID = nID;

		// Calculate the size of the band
		BOOL bRet;
		RECT rcTmp;
		int nBtnCount = (int)::SendMessage(hWndBand, TB_BUTTONCOUNT, 0, 0L);
		if(nBtnCount > 0)
		{
			bRet = (BOOL)::SendMessage(hWndBand, TB_GETITEMRECT, nBtnCount - 1, (LPARAM)&rcTmp);
			ATLASSERT(bRet);
			rbBand.cx = (cxWidth != 0) ? cxWidth : rcTmp.right;
			rbBand.cyMinChild = rcTmp.bottom - rcTmp.top;

			if(bFullWidthAlways)
			{
				rbBand.cxMinChild = rbBand.cx;
			}
			else if(lpstrTitle == 0)
			{
				CRect rcTmp;// check!!
				bRet = (BOOL)::SendMessage(hWndBand, TB_GETITEMRECT, 0, (LPARAM)&rcTmp);
				ATLASSERT(bRet);
				rbBand.cxMinChild = rcTmp.right;
			}
			else
			{
				rbBand.cxMinChild = 0;
			}
			rbBand.cxMinChild = 0; // UDT DGSTR ( force min size
		}
		else	// no buttons, either not a toolbar or really has no buttons
		{
			bRet = ::GetWindowRect(hWndBand, &rcTmp);
			ATLASSERT(bRet);
			rbBand.cx = (cxWidth != 0) ? cxWidth : (rcTmp.right - rcTmp.left);
			rbBand.cxMinChild = (bFullWidthAlways) ? rbBand.cx : 0;
			rbBand.cyMinChild = rcTmp.bottom - rcTmp.top;
		}

#if (_WIN32_IE >= 0x0400)
		// NOTE: cxIdeal used for CHEVRON, if MDI cxIdeal changed dynamically.
		rbBand.cxIdeal = rcTmp.right;//rbBand.cx is not good.
#endif //(_WIN32_IE >= 0x0400)

		// Add the band
		LRESULT lRes = ::SendMessage(hWndReBar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbBand);
		if(lRes == 0)
		{
			ATLTRACE2(atlTraceUI, 0, _T("Failed to add a band to the rebar.\n"));
			return FALSE;
		}

#if (_WIN32_IE >= 0x0501)
		if (nID == IDC_LINKBAR)
			::SendMessage(hWndBand, TB_SETEXTENDEDSTYLE, 0, TBSTYLE_EX_HIDECLIPPEDBUTTONS);
#endif //(_WIN32_IE >= 0x0501)

		return TRUE;
	}

	struct _Function_EnumChildSaveOption
	{
		int _nIndex;
		CString _strFileName;
		HWND _hWndActive;
		int _nActiveIndex;
		_Function_EnumChildSaveOption(const CString& strFileName, HWND hWndActive, int nIndex = 0)
			: _nIndex(nIndex), _strFileName(strFileName),
			  _hWndActive(hWndActive), _nActiveIndex(-1)
		{
		}
		void operator()(HWND hWnd)
		{
			if (hWnd == _hWndActive)
				_nActiveIndex = _nIndex;				

			::SendMessage(hWnd, WM_SAVE_OPTION, (WPARAM)(LPCTSTR)_strFileName, (LPARAM)_nIndex);
			++_nIndex;
		}
	};

	void _SaveGroupOption(const CString& strFileName)
	{
		::DeleteFile(strFileName);// clean up

		BOOL bMaximized = FALSE;
		HWND hWndActive = MDIGetActive(&bMaximized);
		_Function_EnumChildSaveOption f(strFileName, hWndActive);
		f = m_MDITab.ForEachWindow(f);// MtlForEachMDIChild(m_hWndMDIClient, f);

		CIniSection pr;
		pr.Open(strFileName, _T("Header"));
		pr.SetValue(f._nIndex, _T("count"));
		pr.SetValue(f._nActiveIndex, _T("active"));// active index
		pr.SetValue(bMaximized ? 1 : 0, _T("maximized"));
		pr.Close();
	}

	void _LoadGroupOption(const CString& strFileName, bool bClose)
	{
		dmfTRACE(_T("CMainFrame::_LoadGroupOption\n"));

		CLockRedrawMDIClient lock(m_hWndMDIClient);
		CMDITabCtrl::CLockRedraw lock2(m_MDITab);
		CWaitCursor cur;

		if (bClose)
			MtlCloseAllMDIChildren(m_hWndMDIClient);

		CIniSection pr;
		pr.Open(strFileName, _T("Header"));

		DWORD dwCount = 0;
		if (pr.QueryValue(dwCount, _T("count")) != ERROR_SUCCESS)
			return;

		DWORD dwActive = 0;
		pr.QueryValue(dwActive, _T("active"));

		DWORD dwMaximized = 0;
		pr.QueryValue(dwMaximized, _T("maximized"));

		bool bActiveChildExistAlready = (MDIGetActive() == NULL) ? false : true;

		m_MDITab.m_bInsertHere = true;
		m_MDITab.m_nInsertIndex = m_MDITab.GetItemCount();

		CChildFrame* pChildActive = NULL;
		for (DWORD dw = 0; dw < dwCount; ++dw)
		{
			CChildFrame* pChild =
				CChildFrame::NewWindow(m_hWndMDIClient, m_MDITab, m_AddressBar);
				// if no active child, as there is no client edge in MDI client window,
				// so GetClientRect is different a little and a resizing will occur when switching.
				// That is, only the first child window is activated.

			if (pChild == NULL)
				continue;

			++m_MDITab.m_nInsertIndex;

			// activate now!
			pChild->ActivateFrame(MDIGetActive() != NULL ? SW_SHOWNOACTIVATE : -1);

			// if tab mode, no need to load window placement.
			pChild->m_view.m_ViewOption.GetProfile(strFileName, dw, !CMainOption::s_bTabMode);

			if (dw == dwActive)
				pChildActive = pChild;
		}

		m_MDITab.m_bInsertHere = false;
		m_MDITab.m_nInsertIndex = -1;

		if (pChildActive == NULL)
			return;

		if (!bActiveChildExistAlready) {	// there was no active window
			MDIActivate(pChildActive->m_hWnd);
			if (CMainOption::s_bTabMode || dwMaximized == 1) {	// in tab mode
				MDIMaximize(pChildActive->m_hWnd);
			}
		}
		else {								// already an active window exists
			if (!(CMainOption::s_dwMainExtendedStyle & MAIN_EX_NOACTIVATE))
				MDIActivate(pChildActive->m_hWnd);
		}
	}

// UDT DGSTR
	bool _IsExistHTMLHelp(void)
	{
		return ( 0xFFFFFFFF != GetFileAttributes(MtlGetHTMLHelpPath()) ); 
	}
// ENDE

	BEGIN_UPDATE_COMMAND_UI_MAP(CMainFrame)
		CHAIN_UPDATE_COMMAND_UI_MEMBER(m_MainOption)
//		CHAIN_UPDATE_COMMAND_UI_MEMBER(m_secZone)
		CHAIN_UPDATE_COMMAND_UI_MEMBER(m_ExplorerBar)
		CHAIN_MDI_CHILD_UPDATE_COMMAND_UI()				// first of all

		UPDATE_COMMAND_UI_ENABLE_SETCHECK_FLAG_REV(ID_DLCTL_COOKIES ,URLACTION_COOKIES,GetSecure(URLACTION_COOKIES)) // UDT DGSTR
		UPDATE_COMMAND_UI_ENABLE_SETCHECK_FLAG_REV(ID_DLCTL_COOKIES_SESSION,URLACTION_COOKIES_SESSION,GetSecure(URLACTION_COOKIES_SESSION)) // UDT DGSTR

		bool bActiveChild = (MDIGetActive() != NULL);
		// File menu
			UPDATE_COMMAND_UI_SETDEFAULT_FLAG(ID_FILE_NEW_HOME, FILENEW_HOME, CFileNewOption::s_dwFlags)
			UPDATE_COMMAND_UI_SETDEFAULT_FLAG_PASS(ID_FILE_NEW_COPY, FILENEW_COPY, CFileNewOption::s_dwFlags)
			UPDATE_COMMAND_UI_SETDEFAULT_FLAG(ID_FILE_NEW_BLANK, FILENEW_BLANK, CFileNewOption::s_dwFlags)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_NEW_COPY, bActiveChild)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_NEW_CLIPBOARD, _IsClipboardAvailable())
			UPDATE_COMMAND_UI_SETDEFAULT_PASS(ID_FILE_NEW_CLIPBOARD2)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_NEW_CLIPBOARD2, _IsClipboardAvailable())
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_SAVE_AS, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_PAGE_SETUP, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_PRINT, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_PRINTPREVIEW, bActiveChild) // UDT DGSTR
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_PROPERTIES, bActiveChild)
		UPDATE_COMMAND_UI_SETCHECK_IF(ID_FILE_WORKOFFLINE, MtlIsGlobalOffline())
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FILE_CLOSE, bActiveChild)
		// Edit menu
//		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_CUT, bActiveChild)
//		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_COPY, bActiveChild)
//		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_PASTE, bActiveChild)
//		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_SELECT_ALL, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_FIND, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_FIND_MAX, bActiveChild) // UDT DGSTR
		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_TITLE_COPY, bActiveChild) // UDT DGSTR
		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_URL_COPY, bActiveChild) // UDT DGSTR
		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_OPEN_SELECTED, bActiveChild) // UDT DGSTR
		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_OPEN_LINK_ALL, bActiveChild) // UDT DGSTR
//		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_SETFOCUS, bActiveChild)
		// View menu
			UPDATE_COMMAND_UI_SETCHECK_IF(ID_VIEW_TOOLBAR, MtlIsBandVisible(m_hWndToolBar, ATL_IDW_TOOLBAR))
			UPDATE_COMMAND_UI_SETCHECK_IF(ID_VIEW_ADDRESSBAR, MtlIsBandVisible(m_hWndToolBar, IDC_ADDRESSBAR))
			UPDATE_COMMAND_UI_SETCHECK_IF(ID_VIEW_LINKBAR, MtlIsBandVisible(m_hWndToolBar, IDC_LINKBAR))
			UPDATE_COMMAND_UI_SETCHECK_FLAG(ID_VIEW_GOBUTTON, ABR_EX_GOBTNVISIBLE, m_AddressBar.GetAddressBarExtendedStyle())
			UPDATE_COMMAND_UI_SETCHECK_IF(ID_VIEW_TABBAR, MtlIsBandVisible(m_hWndToolBar, IDC_MDITAB))
			UPDATE_COMMAND_UI_SETCHECK_FLAG(ID_VIEW_TABBAR_MULTI, MTB_EX_MULTILINE, m_MDITab.GetMDITabExtendedStyle())
		UPDATE_COMMAND_UI_SETCHECK_IF(ID_VIEW_STATUS_BAR, ::IsWindowVisible(m_hWndStatusBar))

		UPDATE_COMMAND_UI_ENABLE_IF_WITH_POPUP(ID_VIEW_BACK, bActiveChild, bActiveChild)		// with popup
			UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_FORWARD, bActiveChild)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_REFRESH, bActiveChild)

		UPDATE_COMMAND_UI_POPUP_ENABLE_IF(ID_VIEW_BACK1, bActiveChild)
		UPDATE_COMMAND_UI_POPUP_ENABLE_IF(ID_VIEW_FORWARD1, bActiveChild)

		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_STOP, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_STOP_ALL, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_REFRESH, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_REFRESH_ALL, bActiveChild)
		UPDATE_COMMAND_UI_POPUP_ENABLE_IF(ID_AUTOREFRESH_NONE, bActiveChild)
		UPDATE_COMMAND_UI_POPUP_ENABLE_IF(ID_VIEW_FONT_LARGEST, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_FONT_SIZE, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_SOURCE, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_SOURCE_SELECTED, bActiveChild) // UDT DGSTR
		UPDATE_COMMAND_UI_SETCHECK_IF(ID_VIEW_FULLSCREEN, _IsFullScreen())

		// Favorite menu
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FAVORITE_ADD, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FAVORITE_GROUP_ADD, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_FAVORITE_GROUP_SAVE, bActiveChild)
		// Tool menu
		UPDATE_COMMAND_UI_ENABLE_IF_WITH_POPUP(ID_DLCTL_DLIMAGES, bActiveChild, bActiveChild)	// with poup
			UPDATE_COMMAND_UI_ENABLE_IF(ID_DLCTL_ALLCHANGEM, bActiveChild) // UDT DGSTR
			UPDATE_COMMAND_UI_ENABLE_IF(ID_DLCTL_BGSOUNDS, bActiveChild)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_DLCTL_VIDEOS, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF_WITH_POPUP(ID_DLCTL_RUNACTIVEXCTLS, bActiveChild, bActiveChild)// with popup
			UPDATE_COMMAND_UI_ENABLE_IF(ID_DLCTL_ALLCHANGES, bActiveChild) // UDT DGSTR
			UPDATE_COMMAND_UI_ENABLE_IF(ID_DLCTL_DLACTIVEXCTLS, bActiveChild)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_DLCTL_SCRIPTS, bActiveChild)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_DLCTL_JAVA, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_DOCHOSTUI_OPENNEWWIN, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_IGNORE, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_EDIT_CLOSE_TITLE, bActiveChild) // UDT DGSTR
		// Window menu
		UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_CLOSE_ALL, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_CLOSE_EXCEPT, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_TAB_LEFT, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_TAB_RIGHT, bActiveChild)
		UPDATE_COMMAND_UI_ENABLE_IF_WITH_POPUP(ID_WINDOW_RESTORE, bActiveChild, bActiveChild)// with popup
			UPDATE_COMMAND_UI_ENABLE_IF(ATL_IDS_SCPREVWINDOW, bActiveChild)
			UPDATE_COMMAND_UI_ENABLE_IF(ATL_IDS_SCNEXTWINDOW, bActiveChild)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_CASCADE, bActiveChild && !CMainOption::s_bTabMode)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_TILE_HORZ, bActiveChild && !CMainOption::s_bTabMode)
			UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_TILE_VERT, bActiveChild && !CMainOption::s_bTabMode)
			UPDATE_COMMAND_UI(ID_WINDOW_ARRANGE, OnUpdateWindowArrange) // to update Window Menu
			// UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_ARRANGE, bActiveChild && !CMainOption::s_bTabMode)
		// Help
		UPDATE_COMMAND_UI_ENABLE_IF(ID_APP_HELP, _IsExistHTMLHelp()) // UDT DGSTR
		// Misc
//		UPDATE_COMMAND_UI_SETTEXT(ID_DEFAULT_PANE, _T("fB"))
		UPDATE_COMMAND_UI(IDC_PROGRESS, OnUpdateProgressUI)
	END_UPDATE_COMMAND_UI_MAP()

	void OnUpdateWindowArrange(CCmdUI* pCmdUI)
	{
		// toolbar has not this command button, called only on menu popuping
		MDIRefreshMenu();
		MtlCompactMDIWindowMenuDocumentList(m_menuWindow, _nWindowMenuItemCount,
			CFavoritesMenuOption::GetMaxMenuItemTextLength());

		pCmdUI->Enable(MDIGetActive() != NULL && !CMainOption::s_bTabMode);
	}

	void OnUpdateProgressUI(CCmdUI* pCmdUI)
	{
		CProgressBarCtrl progressbar = pCmdUI->m_wndOther;
		progressbar.ShowWindow(SW_HIDE);
	}

	bool _IsClipboardAvailable()
	{
		return ::IsClipboardFormatAvailable(CF_TEXT) == TRUE;

/*		CComPtr<IDataObject> spDataObject;
		HRESULT hr = ::OleGetClipboard(&spDataObject);
		if (FAILED(hr))
			return false;

		FORMATETC formatetc = { CF_TEXT, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
		hr = spDataObject->QueryGetData(&formatetc);
		if (hr == NOERROR) {
			return true;
		}
	
		return false;
*/
	}

	void _FocusChecker()
	{
		// Note. On idle time, give focus back to browser window...
		dmfTRACE(_T("_FocusChecker\n"));

		HWND hWndActive = MDIGetActive();
		if (hWndActive == NULL)
			return;

		HWND hWndForeground = ::GetForegroundWindow();
		bool bMysteriousWindowForeground = MtlIsKindOf(hWndForeground, _T("Internet Explorer_Hidden"));
		if (bMysteriousWindowForeground && MtlIsWindowCurrentProcess(hWndForeground))
			MtlSetForegroundWindow(m_hWnd);

		if (!MtlIsApplicationActive(m_hWnd))
		{
			dmfTRACE(_T(" not family(foreground=%x)\n"), hWndForeground);
			return;
		}

		HWND hWndFocus = ::GetFocus();
		if (hWndFocus != NULL && !MtlIsFamily(m_hWnd, hWndFocus)) {
			dmfTRACE(_T(" not family(focus=%x)\n"), hWndFocus);
			return;
		}

		if (::IsChild(m_hWndToolBar, hWndFocus)) {
			dmfTRACE(_T(" on rebar\n"));
			return;
		}

		if (::IsChild(m_ExplorerBar, hWndFocus))
		{
			dmfTRACE(_T(" on explorer bar\n"));
			return;
		}

		if (hWndFocus == NULL || m_hWnd == hWndFocus || hWndActive == hWndFocus ||
			MtlIsKindOf(hWndFocus, _T("Shell DocObject View")))
		{
			dmfTRACE(_T(" Go!!!!!\n"));
			MtlSendCommand(hWndActive, ID_VIEW_SETFOCUS);
		}
	}
};

/////////////////////////////////////////////////////////////////////////////
